@@ -14,6 +14,7 @@
 #include <string.h>
 
 #include "num/multind.h"
+#include "num/init.h"
 
 #include "misc/mmio.h"
 #include "misc/misc.h"
@@ -23,8 +24,8 @@
 #define DIMS 16
 #endif
 
-static const char* usage_str = "dim shift <input> <output>";
-static const char* help_str = "Perform circular shift along {dim} by {shift} elements.\n";
+static const char usage_str[] = "dim shift <input> <output>";
+static const char help_str[] = "Perform circular shift along {dim} by {shift} elements.\n";
 
 
 
@@ -32,6 +33,8 @@ int main_circshift(int argc, char* argv[])
 {
 	mini_cmdline(argc, argv, 4, usage_str, help_str);
 
+	num_init();
+
 	const int N = DIMS;
 	long dims[N];
 
