@@ -12,6 +12,7 @@
 #include <stdio.h>
 
 #include "num/flpmath.h"
+#include "num/init.h"
 
 #include "misc/mmio.h"
 #include "misc/misc.h"
@@ -20,14 +21,16 @@
 #define DIMS 16
 #endif
 
-static const char* usage_str = "<input> <output>";
-static const char* help_str = "Compute complex conjugate.\n";
+static const char usage_str[] = "<input> <output>";
+static const char help_str[] = "Compute complex conjugate.\n";
 
 
 int main_conj(int argc, char* argv[])
 {
 	mini_cmdline(argc, argv, 2, usage_str, help_str);
 
+	num_init();
+
 	const int N = 16;
 	long dims[N];
 	complex float* idata = load_cfl(argv[1], N, dims);
