@@ -7,13 +7,13 @@
  */
 
 #include <stdlib.h>
-#include <assert.h>
 #include <stdio.h>
 #include <complex.h>
 #include <assert.h>
 
 #include "num/multind.h"
 #include "num/flpmath.h"
+#include "num/init.h"
 
 #include "misc/mmio.h"
 #include "misc/misc.h"
@@ -25,14 +25,16 @@
 #endif
 
 
-static const char* usage_str = "<input> <output";
-static const char* help_str = "Copy phase from <input> to <output>.\n";
+static const char usage_str[] = "<input> <output";
+static const char help_str[] = "Copy phase from <input> to <output>.\n";
 
 
 int main_cpyphs(int argc, char* argv[argc])
 {
 	mini_cmdline(argc, argv, 2, usage_str, help_str);
 
+	num_init();
+
 	long dims[DIMS];
 	
 	complex float* in_data = load_cfl(argv[1], DIMS, dims);
