@@ -14,6 +14,7 @@
 
 #include "num/multind.h"
 #include "num/flpmath.h"
+#include "num/init.h"
 
 #include "misc/mmio.h"
 #include "misc/misc.h"
@@ -25,14 +26,16 @@
 #endif
 
 
-static const char* usage_str = "<input> <output>";
-static const char* help_str = "Real value.\n";
+static const char usage_str[] = "<input> <output>";
+static const char help_str[] = "Real value.\n";
 
 
 int main_creal(int argc, char* argv[])
 {
 	mini_cmdline(argc, argv, 2, usage_str, help_str);
 
+	num_init();
+
 	long dims[DIMS];
 	
 	complex float* in_data = load_cfl(argv[1], DIMS, dims);
