@@ -13,6 +13,7 @@
 #include <stdio.h>
 
 #include "num/multind.h"
+#include "num/init.h"
 
 #include "misc/mmio.h"
 #include "misc/misc.h"
@@ -22,14 +23,16 @@
 #endif
 
 
-static const char* usage_str = "dimension size <input> <output>";
-static const char* help_str = "Extracts a sub-array corresponding to the central part of {size} along {dimension}\n";
+static const char usage_str[] = "dimension size <input> <output>";
+static const char help_str[] = "Extracts a sub-array corresponding to the central part of {size} along {dimension}\n";
 
 
 int main_crop(int argc, char* argv[])
 {
 	mini_cmdline(argc, argv, 4, usage_str, help_str);
 
+	num_init();
+
 	int N = DIMS;
 	long in_dims[N];
 	long out_dims[N];
