@@ -13,6 +13,7 @@
 #include <string.h>
 
 #include "num/multind.h"
+#include "num/init.h"
 
 #include "misc/mmio.h"
 #include "misc/io.h"
@@ -20,8 +21,8 @@
 #include "noncart/nufft.h"
 
 
-static const char* usage_str = "<traj>";
-static const char* help_str = "Estimate image dimension from non-Cartesian trajectory.\n"
+static const char usage_str[] = "<traj>";
+static const char help_str[] = "Estimate image dimension from non-Cartesian trajectory.\n"
 			"Assume trajectory scaled to -DIM/2 to DIM/2 (ie dk=1/FOV=1)\n";
 
 
@@ -30,6 +31,8 @@ int main_estdims(int argc, char* argv[])
 {
 	mini_cmdline(argc, argv, 1, usage_str, help_str);
 
+	num_init();
+
 	int N = 16;
 
 	long traj_dims[N];
