@@ -13,6 +13,7 @@
 #include <complex.h>
 
 #include "num/multind.h"
+#include "num/init.h"
 
 #include "misc/mmio.h"
 #include "misc/misc.h"
@@ -20,14 +21,16 @@
 
 #define DIMS 16
 
-static const char* usage_str = "dimension start end <input> <output>";
-static const char* help_str = "Extracts a sub-array along {dim} from index {start} to {end}.\n";
+static const char usage_str[] = "dimension start end <input> <output>";
+static const char help_str[] = "Extracts a sub-array along {dim} from index {start} to {end}.\n";
 
 
 int main_extract(int argc, char* argv[])
 {
 	mini_cmdline(argc, argv, 5, usage_str, help_str);
 
+	num_init();
+
 	long in_dims[DIMS];
 	long out_dims[DIMS];
 	
