@@ -1,10 +1,10 @@
 /* Copyright 2013. The Regents of the University of California.
- * Copyright 2015. Martin Uecker.
+ * Copyright 2015-2016. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors: 
- * 2012, 2015 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2012-2016 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  */
 
 #include <stdbool.h>
@@ -13,6 +13,7 @@
 
 #include "num/multind.h"
 #include "num/fft.h"
+#include "num/init.h"
 
 #include "misc/mmio.h"
 #include "misc/opts.h"
@@ -23,8 +24,8 @@
 #endif
 
 
-static const char* usage_str = "bitmask <input> <output>";
-static const char* help_str = "Performs a fast Fourier transform (FFT) along selected dimensions.";
+static const char usage_str[] = "bitmask <input> <output>";
+static const char help_str[] = "Performs a fast Fourier transform (FFT) along selected dimensions.";
 
 
 
@@ -36,12 +37,14 @@ int main_fft(int argc, char* argv[])
 
 	const struct opt_s opts[] = {
 
-		{ 'u', false, opt_set, &unitary, "\tunitary" },
-		{ 'i', false, opt_set, &inv, "\tinverse" },
+		OPT_SET('u', &unitary, "unitary"),
+		OPT_SET('i', &inv, "inverse"),
 	};
 
 	cmdline(&argc, argv, 3, 3, usage_str, help_str, ARRAY_SIZE(opts), opts);
 
+	num_init();
+
 	long dims[DIMS];
 	complex float* idata = load_cfl(argv[2], DIMS, dims);
 	complex float* data = create_cfl(argv[3], DIMS, dims);
