@@ -14,6 +14,7 @@
 
 #include "num/multind.h"
 #include "num/fft.h"
+#include "num/init.h"
 
 #include "misc/mmio.h"
 #include "misc/misc.h"
@@ -22,8 +23,8 @@
 #define DIMS 16
 #endif
 
-static const char* usage_str = "bitmask <input> <output>";
-static const char* help_str =	"Apply fftshift along dimensions selected by the {bitmask}.\n";
+static const char usage_str[] = "bitmask <input> <output>";
+static const char help_str[] =	"Apply fftshift along dimensions selected by the {bitmask}.\n";
 
 
 
@@ -32,6 +33,8 @@ int main_fftshift(int argc, char* argv[])
 {
 	bool b = mini_cmdline_bool(argc, argv, 'b', 3, usage_str, help_str);
 
+	num_init();
+
 	unsigned long flags = labs(atol(argv[1]));
 
 	int N = DIMS;
