@@ -13,6 +13,7 @@
 #include <stdio.h>
 
 #include "num/multind.h"
+#include "num/init.h"
 
 #include "misc/mmio.h"
 #include "misc/misc.h"
@@ -22,8 +23,8 @@
 #endif
 
 
-static const char* usage_str = "bitmask <input> <output>";
-static const char* help_str = "Flip (reverse) dimensions specified by the {bitmask}.\n";
+static const char usage_str[] = "bitmask <input> <output>";
+static const char help_str[] = "Flip (reverse) dimensions specified by the {bitmask}.\n";
 
 
 
@@ -31,6 +32,8 @@ int main_flip(int argc, char* argv[])
 {
 	mini_cmdline(argc, argv, 3, usage_str, help_str);
 
+	num_init();
+
 	int N = DIMS;
 	long dims[N];
 	complex float* idata = load_cfl(argv[2], N, dims);
