@@ -14,14 +14,9 @@
 #include <assert.h>
 #include <math.h>
 
-#define  NUM_INTERNAL
-
 #include "num/multind.h"
 #include "num/flpmath.h"
 #include "num/ops.h"
-#ifdef USE_CUDA
-#include "num/gpuops.h"
-#endif
 
 #include "linops/linop.h"
 #include "linops/someops.h"
@@ -38,16 +33,27 @@
 #include "iter.h"
 
 
+DEF_TYPEID(iter_conjgrad_conf);
+DEF_TYPEID(iter_landweber_conf);
+DEF_TYPEID(iter_ist_conf);
+DEF_TYPEID(iter_fista_conf);
+DEF_TYPEID(iter_pocs_conf);
+DEF_TYPEID(iter_admm_conf);
+DEF_TYPEID(iter_call_s);
+
 const struct iter_conjgrad_conf iter_conjgrad_defaults = {
 
+	.INTERFACE.TYPEID = &TYPEID(iter_conjgrad_conf),
+
 	.maxiter = 50,
 	.l2lambda = 0.,
 	.tol = 0.,
 };
 
-
 const struct iter_landweber_conf iter_landweber_defaults = {
 
+	.INTERFACE.TYPEID = &TYPEID(iter_landweber_conf),
+
 	.maxiter = 50,
 	.step = 0.95,
 	.tol = 0.,
@@ -55,6 +61,8 @@ const struct iter_landweber_conf iter_landweber_defaults = {
 
 const struct iter_ist_conf iter_ist_defaults = {
 
+	.INTERFACE.TYPEID = &TYPEID(iter_ist_conf),
+
 	.maxiter = 50,
 	.step = 0.95,
 	.continuation = 1.,
@@ -62,9 +70,10 @@ const struct iter_ist_conf iter_ist_defaults = {
 	.tol = 0.,
 };
 
-
 const struct iter_fista_conf iter_fista_defaults = {
 
+	.INTERFACE.TYPEID = &TYPEID(iter_fista_conf),
+
 	.maxiter = 50,
 	.step = 0.95,
 	.continuation = 1.,
@@ -75,9 +84,13 @@ const struct iter_fista_conf iter_fista_defaults = {
 
 const struct iter_admm_conf iter_admm_defaults = {
 
+	.INTERFACE.TYPEID = &TYPEID(iter_admm_conf),
+
 	.maxiter = 50,
 	.maxitercg = 10,
 
+	.cg_eps = 1.E-3,
+
 	.do_warmstart = false,
 	.dynamic_rho = false,
 	.hogwild = false,
@@ -96,6 +109,8 @@ const struct iter_admm_conf iter_admm_defaults = {
 
 const struct iter_pocs_conf iter_pocs_defaults = {
 
+	.INTERFACE.TYPEID = &TYPEID(iter_pocs_conf),
+
 	.maxiter = 50,
 };
 
@@ -122,29 +137,25 @@ static bool checkeps(float eps)
 
 
 
-void iter_conjgrad(void* _conf,
+void iter_conjgrad(iter_conf* _conf,
 		const struct operator_s* normaleq_op,
 		const struct operator_p_s* thresh_prox,
 		long size, float* image, const float* image_adj,
-		const float* image_truth,
-		void* objval_data,
-		float (*obj_eval)(const void*, const float*))
+		struct iter_monitor_s* monitor)
 {
 	assert(NULL == thresh_prox);
-	iter2_conjgrad(_conf, normaleq_op, 0, NULL, NULL, NULL, size, image, image_adj, image_truth, objval_data, obj_eval);
+	iter2_conjgrad(_conf, normaleq_op, 0, NULL, NULL, NULL, NULL, size, image, image_adj, monitor);
 }
 
 
 
-void iter_landweber(void* _conf,
+void iter_landweber(iter_conf* _conf,
 		const struct operator_s* normaleq_op,
 		const struct operator_p_s* thresh_prox,
 		long size, float* image, const float* image_adj,
-		const float* image_truth,
-		void* objval_data,
-		float (*obj_eval)(const void*, const float*))
+		struct iter_monitor_s* monitor)
 {
-	struct iter_landweber_conf* conf = _conf;
+	struct iter_landweber_conf* conf = CAST_DOWN(iter_landweber_conf, _conf);
 
 	float eps = md_norm(1, MD_DIMS(size), image_adj);
 
@@ -153,11 +164,7 @@ void iter_landweber(void* _conf,
 
 	assert(NULL == thresh_prox);
 
-	UNUSED(obj_eval);
-	UNUSED(objval_data);
-	UNUSED(image_truth);
-
-	landweber_sym(conf->maxiter, 1.E-3 * eps, conf->step, size, (void*)normaleq_op, select_vecops(image_adj), operator_iter, image, image_adj);
+	landweber_sym(conf->maxiter, 1.E-3 * eps, conf->step, size, (void*)normaleq_op, select_vecops(image_adj), operator_iter, image, image_adj, monitor);
 
 cleanup:
 	;
@@ -166,57 +173,50 @@ cleanup:
 
 
 
-void iter_ist(void* _conf,
+void iter_ist(iter_conf* _conf,
 		const struct operator_s* normaleq_op,
 		const struct operator_p_s* thresh_prox,
 		long size, float* image, const float* image_adj,
-		const float* image_truth,
-		void* objval_data,
-		float (*obj_eval)(const void*, const float*))
+		struct iter_monitor_s* monitor)
 {
-	iter2_ist(_conf, normaleq_op, 1, &thresh_prox, NULL, NULL, size, image, image_adj, image_truth, objval_data, obj_eval);
+	iter2_ist(_conf, normaleq_op, 1, &thresh_prox, NULL, NULL, NULL, size, image, image_adj, monitor);
 }
 
-void iter_fista(void* _conf,
+void iter_fista(iter_conf* _conf,
 		const struct operator_s* normaleq_op,
 		const struct operator_p_s* thresh_prox,
 		long size, float* image, const float* image_adj,
-		const float* image_truth,
-		void* objval_data,
-		float (*obj_eval)(const void*, const float*))
+		struct iter_monitor_s* monitor)
 {
-	iter2_fista(_conf, normaleq_op, 1, &thresh_prox, NULL, NULL, size, image, image_adj, image_truth, objval_data, obj_eval);
+	iter2_fista(_conf, normaleq_op, 1, &thresh_prox, NULL, NULL, NULL, size, image, image_adj, monitor);
 }
 
 
 
-void iter_admm(void* _conf,
+void iter_admm(iter_conf* _conf,
 		const struct operator_s* normaleq_op,
 		const struct operator_p_s* thresh_prox,
 		long size, float* image, const float* image_adj,
-		const float* image_truth,
-		void* objval_data,
-		float (*obj_eval)(const void*, const float*))
+		struct iter_monitor_s* monitor)
 {
 	const struct linop_s* eye[1] = { linop_identity_create(1, MD_DIMS(size / 2)) }; // using complex float identity operator... divide size by 2
 
-	iter2_admm(_conf, normaleq_op, 1, &thresh_prox, eye, NULL, size, image, image_adj, image_truth, objval_data, obj_eval);
+	iter2_admm(_conf, normaleq_op, 1, &thresh_prox, eye, NULL, NULL, size, image, image_adj, monitor);
 
 	linop_free(eye[0]);
 }
 
 
-void iter_call_iter2(void* _conf,
+void iter_call_iter2(iter_conf* _conf,
 		const struct operator_s* normaleq_op,
 		const struct operator_p_s* thresh_prox,
 		long size, float* image, const float* image_adj,
-		const float* image_truth,
-		void* objval_data,
-		float (*obj_eval)(const void*, const float*))
+		struct iter_monitor_s* monitor)
 {
-	struct iter2_call_s* it = _conf;
-	it->fun(it->_conf, normaleq_op, (NULL == thresh_prox) ? 1 : 0, &thresh_prox, NULL, NULL,
-		size, image, image_adj, image_truth, objval_data, obj_eval);
+	struct iter2_call_s* it = CAST_DOWN(iter2_call_s, _conf);
+
+	it->fun(it->_conf, normaleq_op, (NULL == thresh_prox) ? 1 : 0, &thresh_prox, NULL, NULL, NULL,
+		size, image, image_adj, monitor);
 }
 
 
