@@ -1,25 +1,35 @@
-/* Copyright 2013-2014. The Regents of the University of California.
- * All rights reserved. Use of this source code is governed by 
+/* Copyright 2013-2016. The Regents of the University of California.
+ * Copyright 2016. Martin Uecker.
+ * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  */
  
 #ifndef __ITER2_H
 #define __ITER2_H
 
+#include "misc/cppwrap.h"
+#include "misc/types.h"
+
 struct linop_s;
 struct operator_s;
 struct operator_p_s;
 
-typedef void (italgo_fun2_f)(void* conf,
+#ifndef ITER_CONF_S
+#define ITER_CONF_S
+typedef struct iter_conf_s { TYPEID* TYPEID; } iter_conf;
+#endif
+
+struct iter_monitor_s;
+
+typedef void (italgo_fun2_f)(iter_conf* conf,
 		const struct operator_s* normaleq_op,
 		unsigned int D,
-		const struct operator_p_s** prox_ops,
-		const struct linop_s** ops,
+		const struct operator_p_s* prox_ops[__VLA2(D)],
+		const struct linop_s* ops[__VLA2(D)],
+		const float* biases[__VLA2(D)],
 		const struct operator_p_s* xupdate_op,
 		long size, float* image, const float* image_adj,
-		const float* image_truth,
-		void* obj_eval_data,
-		float (*obj_eval)(const void*, const float*));
+		struct iter_monitor_s* monitor);
 
 typedef italgo_fun2_f* italgo_fun2_t;
 
@@ -33,12 +43,20 @@ italgo_fun2_f iter2_pocs;
 // use with iter_call_s from iter.h as _conf
 italgo_fun2_f iter2_call_iter;
 
+
 struct iter2_call_s {
 
+	INTERFACE(iter_conf);
+
 	italgo_fun2_t fun;
-	void* _conf;
+	iter_conf* _conf;
 };
 
+extern DEF_TYPEID(iter2_call_s);
+
+
+#include "misc/cppwrap.h"
+
 
 #endif
 
