@@ -1,10 +1,10 @@
 /* Copyright 2014. The Regents of the University of California.
- * Copyright 2015. Martin Uecker.
+ * Copyright 2015-2016. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
- * Authors: 
- * 2014-2015 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * Authors:
+ * 2014-2016 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  * 2014 Jonathan Tamir <jtamir@eecs.berkeley.edu>
  */
 
@@ -22,74 +22,113 @@
 #include "num/conv.h"
 #include "num/ops.h"
 #include "num/iovec.h"
-#include "num/lapack.h"
+#include "num/blas.h"
+#ifdef USE_CUDA
+#include "num/gpuops.h"
+#endif
 
 #include "linops/linop.h"
 
 #include "someops.h"
 
+DEF_TYPEID(cdiag_s);
+
 struct cdiag_s {
 
+	INTERFACE(linop_data_t);
+
 	unsigned int N;
 	const long* dims;
 	const long* strs;
 	const long* dstrs;
 	const complex float* diag;
+#ifdef USE_CUDA
+	const complex float* gpu_diag;
+#endif
 	bool rmul;
 };
 
-static void cdiag_apply(const void* _data, complex float* dst, const complex float* src)
+
+static void cdiag_apply(const linop_data_t* _data, complex float* dst, const complex float* src)
 {
-	const struct cdiag_s* data = _data;
-	(data->rmul ? md_zrmul2 : md_zmul2)(data->N, data->dims, data->strs, dst, data->strs, src, data->dstrs, data->diag);
+	const struct cdiag_s* data = CAST_DOWN(cdiag_s, _data);
+
+	const complex float* diag = data->diag;
+#ifdef USE_CUDA
+	if (cuda_ondevice(src)) {
+
+		if (NULL == data->gpu_diag)
+			((struct cdiag_s*)data)->gpu_diag = md_gpu_move(data->N, data->dims, data->diag, CFL_SIZE);
+
+		diag = data->gpu_diag;
+	}
+#endif
+	(data->rmul ? md_zrmul2 : md_zmul2)(data->N, data->dims, data->strs, dst, data->strs, src, data->dstrs, diag);
 }
 
-static void cdiag_adjoint(const void* _data, complex float* dst, const complex float* src)
+static void cdiag_adjoint(const linop_data_t* _data, complex float* dst, const complex float* src)
 {
-	const struct cdiag_s* data = _data;
-	(data->rmul ? md_zrmul2 : md_zmulc2)(data->N, data->dims, data->strs, dst, data->strs, src, data->dstrs, data->diag);
+	const struct cdiag_s* data = CAST_DOWN(cdiag_s, _data);
+
+	const complex float* diag = data->diag;
+#ifdef USE_CUDA
+	if (cuda_ondevice(src)) {
+
+		if (NULL == data->gpu_diag)
+			((struct cdiag_s*)data)->gpu_diag = md_gpu_move(data->N, data->dims, data->diag, CFL_SIZE);
+
+		diag = data->gpu_diag;
+	}
+#endif
+	(data->rmul ? md_zrmul2 : md_zmulc2)(data->N, data->dims, data->strs, dst, data->strs, src, data->dstrs, diag);
 }
 
-static void cdiag_normal(const void* _data, complex float* dst, const complex float* src)
+static void cdiag_normal(const linop_data_t* _data, complex float* dst, const complex float* src)
 {
 	cdiag_apply(_data, dst, src);
 	cdiag_adjoint(_data, dst, dst);
 }
 
-static void cdiag_free(const void* _data)
+static void cdiag_free(const linop_data_t* _data)
 {
-	const struct cdiag_s* data = _data;
+	const struct cdiag_s* data = CAST_DOWN(cdiag_s, _data);
+
+#ifdef USE_CUDA
+	md_free((void*)data->gpu_diag);
+#endif
 	free((void*)data->dims);
 	free((void*)data->dstrs);
 	free((void*)data->strs);
 	free((void*)data);
 }
 
-
-
-static struct linop_s* linop_gdiag_create(unsigned int N, const long dims[N], unsigned int flags, const _Complex float* diag, bool rdiag)
+static struct linop_s* linop_gdiag_create(unsigned int N, const long dims[N], unsigned int flags, const complex float* diag, bool rdiag)
 {
-	struct cdiag_s* data = xmalloc(sizeof(struct cdiag_s));
+	PTR_ALLOC(struct cdiag_s, data);
+	SET_TYPEID(cdiag_s, data);
 
 	data->rmul = rdiag;
 
 	data->N = N;
-	long* dims2 = xmalloc(N * sizeof(long));
-	long* dstrs = xmalloc(N * sizeof(long));
-	long* strs = xmalloc(N * sizeof(long));
+	PTR_ALLOC(long[N], dims2);
+	PTR_ALLOC(long[N], dstrs);
+	PTR_ALLOC(long[N], strs);
 
 	long ddims[N];
 	md_select_dims(N, flags, ddims, dims);
-	md_copy_dims(N, dims2, dims);
-	md_calc_strides(N, strs, dims, CFL_SIZE);
-	md_calc_strides(N, dstrs, ddims, CFL_SIZE);
+	md_copy_dims(N, *dims2, dims);
+	md_calc_strides(N, *strs, dims, CFL_SIZE);
+	md_calc_strides(N, *dstrs, ddims, CFL_SIZE);
 
-	data->dims = dims2;
-	data->strs = strs;
-	data->dstrs = dstrs;
+	data->dims = *PTR_PASS(dims2);
+	data->strs = *PTR_PASS(strs);
+	data->dstrs = *PTR_PASS(dstrs);
 	data->diag = diag;	// make a copy?
+#ifdef USE_CUDA
+	data->gpu_diag = NULL;
+#endif
 
-	return linop_create(N, dims, N, dims, data, cdiag_apply, cdiag_adjoint, cdiag_normal, NULL, cdiag_free);
+	return linop_create(N, dims, N, dims, CAST_UP(PTR_PASS(data)), cdiag_apply, cdiag_adjoint, cdiag_normal, NULL, cdiag_free);
 }
 
 
@@ -102,7 +141,7 @@ static struct linop_s* linop_gdiag_create(unsigned int N, const long dims[N], un
  * @param flags bitmask specifiying the dimensions present in diag
  * @param diag diagonal matrix
  */
-struct linop_s* linop_cdiag_create(unsigned int N, const long dims[N], unsigned int flags, const _Complex float* diag)
+struct linop_s* linop_cdiag_create(unsigned int N, const long dims[N], unsigned int flags, const complex float* diag)
 {
 	return linop_gdiag_create(N, dims, flags, diag, false);
 }
@@ -116,24 +155,37 @@ struct linop_s* linop_cdiag_create(unsigned int N, const long dims[N], unsigned
  * @param flags bitmask specifiying the dimensions present in diag
  * @param diag diagonal matrix
  */
-struct linop_s* linop_rdiag_create(unsigned int N, const long dims[N], unsigned int flags, const _Complex float* diag)
+struct linop_s* linop_rdiag_create(unsigned int N, const long dims[N], unsigned int flags, const complex float* diag)
 {
 	return linop_gdiag_create(N, dims, flags, diag, true);
 }
 
 
-static void identity_apply(const void* _data, complex float* dst, const complex float* src)
+
+struct identity_data_s {
+
+	INTERFACE(linop_data_t);
+
+	const struct iovec_s* domain;
+};
+
+DEF_TYPEID(identity_data_s);
+
+static void identity_apply(const linop_data_t* _data, complex float* dst, const complex float* src)
 {
-	const struct iovec_s* domain = _data;
+	const struct iovec_s* domain = CAST_DOWN(identity_data_s, _data)->domain;
+
 	md_copy2(domain->N, domain->dims, domain->strs, dst, domain->strs, src, CFL_SIZE);
 }
 
-
-static void identity_free(const void* data)
+static void identity_free(const linop_data_t* _data)
 {	
-	iovec_free((const struct iovec_s*)data);
-}
+	const struct identity_data_s* data = CAST_DOWN(identity_data_s, _data);
 
+	iovec_free(data->domain);
+
+	free((void*)data);
+}
 
 /**
  * Create an Identity linear operator: I x
@@ -142,45 +194,59 @@ static void identity_free(const void* data)
  */
 struct linop_s* linop_identity_create(unsigned int N, const long dims[N])
 {
-	const struct iovec_s* domain = iovec_create(N, dims, CFL_SIZE);
+	PTR_ALLOC(struct identity_data_s, data);
+	SET_TYPEID(identity_data_s, data);
 
-	return linop_create(N, dims, N, dims, (void*)domain, identity_apply, identity_apply, identity_apply, NULL, identity_free);
+	data->domain = iovec_create(N, dims, CFL_SIZE);
+
+	return linop_create(N, dims, N, dims, CAST_UP(PTR_PASS(data)), identity_apply, identity_apply, identity_apply, NULL, identity_free);
 }
 
 
 struct resize_op_s {
 
+	INTERFACE(linop_data_t);
+
 	unsigned int N;
 	const long* out_dims;
 	const long* in_dims;
 };
 
-static void resize_forward(const void* _data, complex float* dst, const complex float* src)
+DEF_TYPEID(resize_op_s);
+
+static void resize_forward(const linop_data_t* _data, complex float* dst, const complex float* src)
 {
-	const struct resize_op_s* data = _data;
+	const struct resize_op_s* data = CAST_DOWN(resize_op_s, _data);
+
 	md_resize_center(data->N, data->out_dims, dst, data->in_dims, src, CFL_SIZE);
 }
 
-static void resize_adjoint(const void* _data, complex float* dst, const complex float* src)
+static void resize_adjoint(const linop_data_t* _data, complex float* dst, const complex float* src)
 {
-	const struct resize_op_s* data = _data;
+	const struct resize_op_s* data = CAST_DOWN(resize_op_s, _data);
+
 	md_resize_center(data->N, data->in_dims, dst, data->out_dims, src, CFL_SIZE);
 }
 
-static void resize_normal(const void* _data, complex float* dst, const complex float* src)
+static void resize_normal(const linop_data_t* _data, complex float* dst, const complex float* src)
 {
-	const struct resize_op_s* data = _data;
+	const struct resize_op_s* data = CAST_DOWN(resize_op_s, _data);
+
 	complex float* tmp = md_alloc_sameplace(data->N, data->out_dims, CFL_SIZE, dst);
+
 	resize_forward(_data, tmp, src);
-	resize_adjoint(_data, dst, (const complex float*)tmp);
+	resize_adjoint(_data, dst, tmp);
+
 	md_free(tmp);
 }
 
-static void resize_free(const void* _data)
+static void resize_free(const linop_data_t* _data)
 {
-	const struct resize_op_s* data = _data;
+	const struct resize_op_s* data = CAST_DOWN(resize_op_s, _data);
+
 	free((void*)data->out_dims);
 	free((void*)data->in_dims);
+
 	free((void*)data);
 }
 
@@ -196,21 +262,24 @@ static void resize_free(const void* _data)
  */
 struct linop_s* linop_resize_create(unsigned int N, const long out_dims[N], const long in_dims[N])
 {
-	struct resize_op_s* data = xmalloc(sizeof(struct resize_op_s));
+	PTR_ALLOC(struct resize_op_s, data);
+	SET_TYPEID(resize_op_s, data);
 
 	data->N = N;
-	data->out_dims = xmalloc(N * sizeof(long));
-	data->in_dims = xmalloc(N * sizeof(long));
+	data->out_dims = *TYPE_ALLOC(long[N]);
+	data->in_dims = *TYPE_ALLOC(long[N]);
 
 	md_copy_dims(N, (long*)data->out_dims, out_dims);
 	md_copy_dims(N, (long*)data->in_dims, in_dims);
 
-	return linop_create(N, out_dims, N, in_dims, data, resize_forward, resize_adjoint, resize_normal, NULL, resize_free);
+	return linop_create(N, out_dims, N, in_dims, CAST_UP(PTR_PASS(data)), resize_forward, resize_adjoint, resize_normal, NULL, resize_free);
 }
 
 
 struct operator_matrix_s {
 
+	INTERFACE(linop_data_t);
+
 	const complex float* mat;
 	const complex float* mat_conj;
 	const complex float* mat_gram; // A^H A
@@ -226,10 +295,9 @@ struct operator_matrix_s {
 	unsigned int K;
 	unsigned int T_dim;
 	unsigned int T;
-
-
 };
 
+DEF_TYPEID(operator_matrix_s);
 
 /**
  * case 1: all singleton dimensions between T_dim and K_dim, all singleton dimensions after K_dim.
@@ -251,6 +319,7 @@ static bool cgemm_forward_standard(const struct operator_matrix_s* data)
 	if (T_dim < K_dim) {
 		
 		for (int i = T_dim + 1; i < N; i++) {
+
 			dsum += data->domain_iovec->dims[i] - 1;
 			csum += data->codomain_iovec->dims[i] - 1;
 			//debug_printf(DP_DEBUG1, "csum = %d, dsum = %d\n", csum, dsum);
@@ -258,20 +327,19 @@ static bool cgemm_forward_standard(const struct operator_matrix_s* data)
 		// don't count K_dim
 		dsum -= data->domain_iovec->dims[K_dim] - 1;
 
-		if (dsum + csum == 0)
+		if ((dsum + csum) == 0)
 			use_cgemm = true;
 	}
 
 	//debug_printf(DP_DEBUG1, "use_cgemm = %d, dsum = %d, csum = %d\n", use_cgemm, dsum, csum);
 
 	return use_cgemm;
-
 }
 
 
-static void linop_matrix_apply(const void* _data, complex float* dst, const complex float* src)
+static void linop_matrix_apply(const linop_data_t* _data, complex float* dst, const complex float* src)
 {
-	const struct operator_matrix_s* data = _data;
+	const struct operator_matrix_s* data = CAST_DOWN(operator_matrix_s, _data);
 
 	long N = data->mat_iovec->N;
 	//debug_printf(DP_DEBUG1, "compute forward\n");
@@ -280,17 +348,24 @@ static void linop_matrix_apply(const void* _data, complex float* dst, const comp
 
 	// FIXME check all the cases where computation can be done with blas
 	
-	if ( cgemm_forward_standard(data) ) {
+	if (cgemm_forward_standard(data)) {
+
 		long L = md_calc_size(data->T_dim, data->domain_iovec->dims);
-		cgemm_sameplace('N', 'T', L, data->T, data->K, &(complex float){1.}, (const complex float (*) [])src, L, (const complex float (*) [])data->mat, data->T, &(complex float){0.}, (complex float (*) [])dst, L);
-	}
-	else
+
+		blas_cgemm('N', 'T', L, data->T, data->K, 1.,
+				L, (const complex float (*)[])src,
+				data->T, (const complex float (*)[])data->mat,
+				0., L, (complex float (*)[])dst);
+
+	} else {
+
 		md_zfmac2(N, data->max_dims, data->codomain_iovec->strs, dst, data->domain_iovec->strs, src, data->mat_iovec->strs, data->mat);
+	}
 }
 
-static void linop_matrix_apply_adjoint(const void* _data, complex float* dst, const complex float* src)
+static void linop_matrix_apply_adjoint(const linop_data_t* _data, complex float* dst, const complex float* src)
 {
-	const struct operator_matrix_s* data = _data;
+	const struct operator_matrix_s* data = CAST_DOWN(operator_matrix_s, _data);
 
 	unsigned int N = data->mat_iovec->N;
 	//debug_printf(DP_DEBUG1, "compute adjoint\n");
@@ -299,23 +374,31 @@ static void linop_matrix_apply_adjoint(const void* _data, complex float* dst, co
 
 	// FIXME check all the cases where computation can be done with blas
 	
-	if ( cgemm_forward_standard(data) ) {
+	if (cgemm_forward_standard(data)) {
+
 		long L = md_calc_size(data->T_dim, data->domain_iovec->dims);
-		cgemm_sameplace('N', 'N', L, data->K, data->T, &(complex float){1.}, (const complex float (*) [])src, L, (const complex float (*) [])data->mat_conj, data->T, &(complex float){0.}, (complex float (*) [])dst, L);
-	}
-	else
+
+		blas_cgemm('N', 'N', L, data->K, data->T, 1.,
+				L, (const complex float (*)[])src,
+				data->T, (const complex float (*)[])data->mat_conj,
+				0., L, (complex float (*)[])dst);
+
+	} else {
+
 		md_zfmacc2(N, data->max_dims, data->domain_iovec->strs, dst, data->codomain_iovec->strs, src, data->mat_iovec->strs, data->mat);
+	}
 }
 
-static void linop_matrix_apply_normal(const void* _data, complex float* dst, const complex float* src)
+static void linop_matrix_apply_normal(const linop_data_t* _data, complex float* dst, const complex float* src)
 {
-	const struct operator_matrix_s* data = _data;
+	const struct operator_matrix_s* data = CAST_DOWN(operator_matrix_s, _data);
 
 	unsigned int N = data->mat_iovec->N;
 	// FIXME check all the cases where computation can be done with blas
 	
 	//debug_printf(DP_DEBUG1, "compute normal\n");
-	if ( cgemm_forward_standard(data) ) {
+	if (cgemm_forward_standard(data)) {
+
 		long max_dims_gram[N];
 		md_copy_dims(N, max_dims_gram, data->domain_iovec->dims);
 		max_dims_gram[data->T_dim] = data->K;
@@ -333,17 +416,22 @@ static void linop_matrix_apply_normal(const void* _data, complex float* dst, con
 		md_transpose(N, data->T_dim, data->K_dim, data->domain_iovec->dims, dst, tmp_dims, tmp, CFL_SIZE);
 
 		md_free(tmp);
-	}
-	else {
+
+	} else {
+
 		long L = md_calc_size(data->T_dim, data->domain_iovec->dims);
-		cgemm_sameplace('N', 'T', L, data->K, data->K, &(complex float){1.}, (const complex float (*) [])src, L, (const complex float (*) [])data->mat_gram, data->K, &(complex float){0.}, (complex float (*) [])dst, L);
+
+		blas_cgemm('N', 'T', L, data->K, data->K, 1.,
+				L, (const complex float (*)[])src,
+				data->K, (const complex float (*)[])data->mat_gram,
+				0., L, (complex float (*)[])dst);
 	}
 
 }
 
-static void linop_matrix_del(const void* _data)
+static void linop_matrix_del(const linop_data_t* _data)
 {
-	const struct operator_matrix_s* data = _data;
+	const struct operator_matrix_s* data = CAST_DOWN(operator_matrix_s, _data);
 
 	iovec_free(data->mat_iovec);
 	iovec_free(data->mat_gram_iovec);
@@ -385,15 +473,15 @@ const struct iovec_s* compute_gram_matrix(unsigned int N, unsigned int T_dim, un
 	// A_dims = [1 K K]  or  [K 1 K]
 	// after: gram_dims = [1 K1 K2] --> [K2 K1 1]  or  [K1 1 K2] --> [K1 K2 1]
 
-	long* A_dims = xmalloc( (N + 1) * sizeof(long) );
-	long* B_dims = xmalloc( (N + 1) * sizeof(long) );
-	long* C_dims = xmalloc( (N + 1) * sizeof(long) );
-	long* fake_gram_dims = xmalloc( (N + 1) * sizeof(long) );
+	long A_dims[N + 1];
+	long B_dims[N + 1];
+	long C_dims[N + 1];
+	long fake_gram_dims[N + 1];
 
-	long* A_str = xmalloc( (N + 1) * sizeof(long) );
-	long* B_str = xmalloc( (N + 1) * sizeof(long) );
-	long* C_str = xmalloc( (N + 1) * sizeof(long) );
-	long* max_dims = xmalloc( (N + 1) * sizeof(long) );
+	long A_str[N + 1];
+	long B_str[N + 1];
+	long C_str[N + 1];
+	long max_dims[N + 1];
 
 	md_singleton_dims(N + 1, A_dims);
 	md_singleton_dims(N + 1, B_dims);
@@ -438,14 +526,6 @@ const struct iovec_s* compute_gram_matrix(unsigned int N, unsigned int T_dim, un
 
 	const struct iovec_s* s =  iovec_create(N, fake_gram_dims, CFL_SIZE);
 
-	free(A_dims);
-	free(B_dims);
-	free(C_dims);
-	free(fake_gram_dims);
-	free(A_str);
-	free(B_str);
-	free(C_str);
-	free(max_dims);
 	md_free(tmpA);
 	md_free(tmpB);
 	md_free(tmpC);
@@ -482,18 +562,23 @@ struct linop_s* linop_matrix_altcreate(unsigned int N, const long out_dims[N], c
 	unsigned int T = out_dims[T_dim];
 	unsigned int K = in_dims[K_dim];
 
-	long* max_dims = xmalloc( N * sizeof(long) );
+	PTR_ALLOC(long[N], max_dims);
 
 	for (unsigned int i = 0; i < N; i++) {
-		if (in_dims[i] > 1 && out_dims[i] == 1) {
-			max_dims[i] = in_dims[i];
+
+		if ((in_dims[i] > 1) && (out_dims[i] == 1)) {
+
+			(*max_dims)[i] = in_dims[i];
 		}
-		else if (in_dims[i] == 1 && out_dims[i] > 1) {
-			max_dims[i] = out_dims[i];
+		else if ((in_dims[i] == 1) && (out_dims[i] > 1)) {
+
+			(*max_dims)[i] = out_dims[i];
 		}
 		else {
+
 			assert(in_dims[i] == out_dims[i]);
-			max_dims[i] = in_dims[i];
+
+			(*max_dims)[i] = in_dims[i];
 		}
 	}
 
@@ -506,12 +591,13 @@ struct linop_s* linop_matrix_altcreate(unsigned int N, const long out_dims[N], c
 	complex float* gram = NULL;
 	const struct iovec_s* gram_iovec = compute_gram_matrix(N, T_dim, T, K_dim, K, &gram, matrix_dims, matrix);
 
-	struct operator_matrix_s* data = xmalloc(sizeof(struct operator_matrix_s));
+	PTR_ALLOC(struct operator_matrix_s, data);
+	SET_TYPEID(operator_matrix_s, data);
 
 	data->mat_iovec = iovec_create(N, matrix_dims, CFL_SIZE);
 	data->mat_gram_iovec = gram_iovec;
 
-	data->max_dims = max_dims;
+	data->max_dims = *max_dims;
 
 	data->mat = mat;
 	data->mat_conj = matc;
@@ -525,7 +611,7 @@ struct linop_s* linop_matrix_altcreate(unsigned int N, const long out_dims[N], c
 	data->domain_iovec = iovec_create(N, in_dims, CFL_SIZE);
 	data->codomain_iovec = iovec_create(N, out_dims, CFL_SIZE);
 
-	return linop_create(N, out_dims, N, in_dims, data, linop_matrix_apply, linop_matrix_apply_adjoint, linop_matrix_apply_normal, NULL, linop_matrix_del);
+	return linop_create(N, out_dims, N, in_dims, CAST_UP(PTR_PASS(data)), linop_matrix_apply, linop_matrix_apply_adjoint, linop_matrix_apply_normal, NULL, linop_matrix_del);
 }
 
 
@@ -557,21 +643,23 @@ struct linop_s* linop_matrix_create(unsigned int N, const long out_dims[N], cons
 
 	for (unsigned int i = 0; i < N; i++) {
 
-		if (in_dims[i] > 1 && out_dims[i] == 1) {
+		if ((in_dims[i] > 1) && (out_dims[i] == 1)) {
+
 			K_dim = i;
 		}
-		else if (in_dims[i] == 1 && out_dims[i] > 1) {
+		else if ((in_dims[i] == 1) && (out_dims[i] > 1)) {
+
 			T_dim = i;
 		}
 		else {
+
 			assert(in_dims[i] == out_dims[i]);
 		}
 	}
 
-	assert(K_dim < (N + 1) && T_dim < (N + 1)); // FIXME what if K_dim or T_dim == 1?
+	assert((K_dim < (N + 1)) && (T_dim < (N + 1))); // FIXME what if K_dim or T_dim == 1?
 
 	return linop_matrix_altcreate(N, out_dims, in_dims, T_dim, K_dim, matrix);
-
 }
 
 
@@ -585,7 +673,6 @@ struct linop_s* linop_matrix_create(unsigned int N, const long out_dims[N], cons
  */
 struct linop_s* linop_matrix_chain(const struct linop_s* a, const struct linop_s* b)
 {
-
 	const struct operator_matrix_s* a_data = linop_get_data(a);
 	const struct operator_matrix_s* b_data = linop_get_data(b);
 
@@ -593,48 +680,43 @@ struct linop_s* linop_matrix_chain(const struct linop_s* a, const struct linop_s
 	assert(linop_codomain(a)->N == linop_domain(b)->N);
 	assert(md_calc_size(linop_codomain(a)->N, linop_codomain(a)->dims) == md_calc_size(linop_domain(b)->N, linop_domain(b)->dims));
 	assert(a_data->K_dim != b_data->T_dim); // FIXME error for now -- need to deal with this specially.
-	assert(a_data->T_dim == b_data->K_dim && a_data->T == b_data->K);
+	assert((a_data->T_dim == b_data->K_dim) && (a_data->T == b_data->K));
 
 	unsigned int N = linop_domain(a)->N;
 
-	long* max_dims = xmalloc( N * sizeof(long) );
+	long max_dims[N];
 
 	md_singleton_dims(N, max_dims);
 	max_dims[a_data->T_dim] = a_data->T;
 	max_dims[a_data->K_dim] = a_data->K;
 	max_dims[b_data->T_dim] = b_data->T;
 
-	long* matrix_dims = xmalloc( N * sizeof(long) );
-	long* matrix_strs = xmalloc( N * sizeof(long) );
+	long matrix_dims[N];
+	long matrix_strs[N];
 
 	md_select_dims(N, ~MD_BIT(a_data->T_dim), matrix_dims, max_dims);
 	md_calc_strides(N, matrix_strs, matrix_dims, CFL_SIZE);
 
 	complex float* matrix = md_alloc_sameplace(N, matrix_dims, CFL_SIZE, a_data->mat);
+
 	md_clear(N, matrix_dims, matrix, CFL_SIZE);
 	md_zfmac2(N, max_dims, matrix_strs, matrix, a_data->mat_iovec->strs, a_data->mat, b_data->mat_iovec->strs, b_data->mat);
 
 	struct linop_s* c = linop_matrix_create(N, linop_codomain(b)->dims, linop_domain(a)->dims, matrix_dims, matrix);
 
-	free(max_dims);
 	md_free(matrix);
-	free(matrix_dims);
-	free(matrix_strs);
-
 
 	return c;
-
 }
 
 
 struct fft_linop_s {
 
+	INTERFACE(linop_data_t);
+
 	const struct operator_s* frw;
 	const struct operator_s* adj;
 
-	complex float* fftmod_mat;
-	complex float* fftmodk_mat;
-
 	bool center;
 	float nscale;
 	
@@ -643,53 +725,31 @@ struct fft_linop_s {
 	long* strs;
 };
 
-static void fft_linop_apply(const void* _data, complex float* out, const complex float* in)
+DEF_TYPEID(fft_linop_s);
+
+static void fft_linop_apply(const linop_data_t* _data, complex float* out, const complex float* in)
 {
-	const struct fft_linop_s* data = _data;
+	const struct fft_linop_s* data = CAST_DOWN(fft_linop_s, _data);
 
-	// fftmod + fftscale
-	if (data->center) {
-
-		md_zmul2(data->N, data->dims, data->strs, out, data->strs, in, data->strs, data->fftmod_mat);
-
-	} else {
-
-		if (in != out)
-			md_copy2(data->N, data->dims, data->strs, out, data->strs, in, CFL_SIZE);
-	}
+	if (in != out)
+		md_copy2(data->N, data->dims, data->strs, out, data->strs, in, CFL_SIZE);
 
 	operator_apply(data->frw, data->N, data->dims, out, data->N, data->dims, out);
-
-	// fftmodk
-	if (data->center)
-		md_zmul2(data->N, data->dims, data->strs, out, data->strs, out, data->strs, data->fftmodk_mat);
 }
 
-static void fft_linop_adjoint(const void* _data, complex float* out, const complex float* in)
+static void fft_linop_adjoint(const linop_data_t* _data, complex float* out, const complex float* in)
 {
-	const struct fft_linop_s* data = _data;
+	const struct fft_linop_s* data = CAST_DOWN(fft_linop_s, _data);
 
-	// fftmod
-	if (data->center) {
-
-		md_zmulc2(data->N, data->dims, data->strs, out, data->strs, in, data->strs, data->fftmodk_mat);
-
-	} else {
-
-		if (in != out)
-			md_copy2(data->N, data->dims, data->strs, out, data->strs, in, CFL_SIZE);
-	}
+	if (in != out)
+		md_copy2(data->N, data->dims, data->strs, out, data->strs, in, CFL_SIZE);
 
 	operator_apply(data->adj, data->N, data->dims, out, data->N, data->dims, out);
-
-	// fftmod + fftscale
-	if (data->center)
-		md_zmulc2(data->N, data->dims, data->strs, out, data->strs, out, data->strs, data->fftmod_mat);
 }
 
-static void fft_linop_free(const void* _data)
+static void fft_linop_free(const linop_data_t* _data)
 {
-	const struct fft_linop_s* data = _data;
+	const struct fft_linop_s* data = CAST_DOWN(fft_linop_s, _data);
 
 	fft_free(data->frw);
 	fft_free(data->adj);
@@ -697,15 +757,12 @@ static void fft_linop_free(const void* _data)
 	free(data->dims);
 	free(data->strs);
 
-	md_free(data->fftmod_mat);
-	md_free(data->fftmodk_mat);
-
 	free((void*)data);
 }
 
-static void fft_linop_normal(const void* _data, complex float* out, const complex float* in)
+static void fft_linop_normal(const linop_data_t* _data, complex float* out, const complex float* in)
 {
-	const struct fft_linop_s* data = _data;
+	const struct fft_linop_s* data = CAST_DOWN(fft_linop_s, _data);
 
 	if (data->center)
 		md_copy(data->N, data->dims, out, in, CFL_SIZE);
@@ -714,81 +771,63 @@ static void fft_linop_normal(const void* _data, complex float* out, const comple
 }
 
 
-static struct linop_s* linop_fft_create_priv(int N, const long dims[N], unsigned int flags, bool gpu, bool forward, bool center)
+static struct linop_s* linop_fft_create_priv(int N, const long dims[N], unsigned int flags, bool forward, bool center)
 {
-#ifdef USE_CUDA
-	md_alloc_fun_t alloc = (gpu ? md_alloc_gpu : md_alloc);
-#else
-	UNUSED(gpu);
-	md_alloc_fun_t alloc = md_alloc;
-#endif
+	const struct operator_s* plan = fft_measure_create(N, dims, flags, true, false);
+	const struct operator_s* iplan = fft_measure_create(N, dims, flags, true, true);
 
-	// FIXME: we allocate only to communicate the gpu flag
-	// and that need in-place plans
-	complex float* tmp = alloc(N, dims, CFL_SIZE);
-	const struct operator_s* plan = fft_create(N, dims, flags, tmp, tmp, false);
-	const struct operator_s* iplan = fft_create(N, dims, flags, tmp, tmp, true);
-	md_free(tmp);
+	PTR_ALLOC(struct fft_linop_s, data);
+	SET_TYPEID(fft_linop_s, data);
 
-	struct fft_linop_s* data = xmalloc(sizeof(struct fft_linop_s));
 	data->frw = plan;
 	data->adj = iplan;
 	data->N = N;
 
 	data->center = center;
 
-	data->dims = xmalloc(N * sizeof(long));
+	data->dims = *TYPE_ALLOC(long[N]);
 	md_copy_dims(N, data->dims, dims);
 
-	data->strs = xmalloc(N * sizeof(long));
+	data->strs = *TYPE_ALLOC(long[N]);
 	md_calc_strides(N, data->strs, data->dims, CFL_SIZE);
 
+	long fft_dims[N];
+	md_select_dims(N, flags, fft_dims, dims);
+	data->nscale = (float)md_calc_size(N, fft_dims);
+
+	lop_fun_t apply = forward ? fft_linop_apply : fft_linop_adjoint;
+	lop_fun_t adjoint = forward ? fft_linop_adjoint : fft_linop_apply;
+
+	struct linop_s* lop =  linop_create(N, dims, N, dims, CAST_UP(PTR_PASS(data)), apply, adjoint, fft_linop_normal, NULL, fft_linop_free);
 
 	if (center) {
 
 		// FIXME: should only allocate flagged dims
 
 		complex float* fftmod_mat = md_alloc(N, dims, CFL_SIZE);
+		complex float* fftmodk_mat = md_alloc(N, dims, CFL_SIZE);
+
+		// we need fftmodk only because we want to apply scaling only once
 
 		complex float one[1] = { 1. };
 		md_fill(N, dims, fftmod_mat, one, CFL_SIZE);
-		fftscale(N, dims, flags, fftmod_mat, fftmod_mat);
-		fftmod(N, dims, flags, fftmod_mat, fftmod_mat);
+		fftmod(N, dims, flags, fftmodk_mat, fftmod_mat);
+		fftscale(N, dims, flags, fftmod_mat, fftmodk_mat);
 
-		// we need it only because we want to apply scaling only once
+		struct linop_s* mod = linop_cdiag_create(N, dims, ~0u, fftmod_mat);
+		struct linop_s* modk = linop_cdiag_create(N, dims, ~0u, fftmodk_mat);
 
-		complex float* fftmodk_mat = md_alloc(N, dims, CFL_SIZE);
+		struct linop_s* tmp = linop_chain(mod, lop);
+		tmp = linop_chain(tmp, modk);
 
-		md_fill(N, dims, fftmodk_mat, one, CFL_SIZE);
-		fftmod(N, dims, flags, fftmodk_mat, fftmodk_mat);
+		linop_free(lop);
+		linop_free(mod);
+		linop_free(modk);
 
-		data->fftmod_mat = fftmod_mat;
-		data->fftmodk_mat = fftmodk_mat;
-
-#ifdef USE_CUDA
-		if (gpu) {
-
-			data->fftmod_mat = md_gpu_move(N, dims, fftmod_mat, CFL_SIZE);
-			data->fftmodk_mat = md_gpu_move(N, dims, fftmodk_mat, CFL_SIZE);
-
-			md_free(fftmod_mat);
-			md_free(fftmodk_mat);
-		}
-#endif
-	} else {
-
-		data->fftmod_mat = NULL;
-		data->fftmodk_mat = NULL;
-
-		long fft_dims[N];
-		md_select_dims(N, flags, fft_dims, dims);
-		data->nscale = (float)md_calc_size(N, fft_dims);
+		lop = tmp;
 	}
 
-	lop_fun_t apply = forward ? fft_linop_apply : fft_linop_adjoint;
-	lop_fun_t adjoint = forward ? fft_linop_adjoint : fft_linop_apply;
-
-	return linop_create(N, dims, N, dims, data, apply, adjoint, fft_linop_normal, NULL, fft_linop_free);
+	return lop;
 }
 
 
@@ -800,9 +839,9 @@ static struct linop_s* linop_fft_create_priv(int N, const long dims[N], unsigned
  * @param flags bitmask of the dimensions to apply the Fourier transform
  * @param gpu use gpu
  */
-struct linop_s* linop_fft_create(int N, const long dims[N], unsigned int flags, bool gpu)
+struct linop_s* linop_fft_create(int N, const long dims[N], unsigned int flags)
 {
-	return linop_fft_create_priv(N, dims, flags, gpu, true, false);
+	return linop_fft_create_priv(N, dims, flags, true, false);
 }
 
 
@@ -812,11 +851,10 @@ struct linop_s* linop_fft_create(int N, const long dims[N], unsigned int flags,
  * @param N number of dimensions
  * @param dims dimensions of input
  * @param flags bitmask of the dimensions to apply the Fourier transform
- * @param gpu use gpu
  */
-struct linop_s* linop_ifft_create(int N, const long dims[N], unsigned int flags, bool gpu)
+struct linop_s* linop_ifft_create(int N, const long dims[N], unsigned int flags)
 {
-	return linop_fft_create_priv(N, dims, flags, gpu, false, false);
+	return linop_fft_create_priv(N, dims, flags, false, false);
 }
 
 
@@ -826,11 +864,10 @@ struct linop_s* linop_ifft_create(int N, const long dims[N], unsigned int flags,
  * @param N number of dimensions
  * @param dims dimensions of input
  * @param flags bitmask of the dimensions to apply the Fourier transform
- * @param gpu use gpu
  */
-struct linop_s* linop_fftc_create(int N, const long dims[N], unsigned int flags, bool gpu)
+struct linop_s* linop_fftc_create(int N, const long dims[N], unsigned int flags)
 {
-	return linop_fft_create_priv(N, dims, flags, gpu, true, true);
+	return linop_fft_create_priv(N, dims, flags, true, true);
 }
 
 
@@ -840,11 +877,10 @@ struct linop_s* linop_fftc_create(int N, const long dims[N], unsigned int flags,
  * @param N number of dimensions
  * @param dims dimensions of input
  * @param flags bitmask of the dimensions to apply the Fourier transform
- * @param gpu use gpu
  */
-struct linop_s* linop_ifftc_create(int N, const long dims[N], unsigned int flags, bool gpu)
+struct linop_s* linop_ifftc_create(int N, const long dims[N], unsigned int flags)
 {
-	return linop_fft_create_priv(N, dims, flags, gpu, false, true);
+	return linop_fft_create_priv(N, dims, flags, false, true);
 }
 
 
@@ -852,38 +888,44 @@ struct linop_s* linop_ifftc_create(int N, const long dims[N], unsigned int flags
 
 struct linop_cdf97_s {
 
+	INTERFACE(linop_data_t);
+
 	unsigned int N;
 	const long* dims;
 	unsigned int flags;
 };
 
-static void linop_cdf97_apply(const void* _data, complex float* out, const complex float* in)
+DEF_TYPEID(linop_cdf97_s);
+
+static void linop_cdf97_apply(const linop_data_t* _data, complex float* out, const complex float* in)
 {
-	const struct linop_cdf97_s* data = _data;
+	const struct linop_cdf97_s* data = CAST_DOWN(linop_cdf97_s, _data);
 
 	md_copy(data->N, data->dims, out, in, CFL_SIZE);
 	md_cdf97z(data->N, data->dims, data->flags, out);
 }
 
-static void linop_cdf97_adjoint(const void* _data, complex float* out, const complex float* in)
+static void linop_cdf97_adjoint(const linop_data_t* _data, complex float* out, const complex float* in)
 {
-	const struct linop_cdf97_s* data = _data;
+	const struct linop_cdf97_s* data = CAST_DOWN(linop_cdf97_s, _data);
 
 	md_copy(data->N, data->dims, out, in, CFL_SIZE);
 	md_icdf97z(data->N, data->dims, data->flags, out);
 }
 
-static void linop_cdf97_normal(const void* _data, complex float* out, const complex float* in)
+static void linop_cdf97_normal(const linop_data_t* _data, complex float* out, const complex float* in)
 {
-	const struct linop_cdf97_s* data = _data;
+	const struct linop_cdf97_s* data = CAST_DOWN(linop_cdf97_s, _data);
 
 	md_copy(data->N, data->dims, out, in, CFL_SIZE);
 }
 
-static void linop_cdf97_free(const void* _data)
+static void linop_cdf97_free(const linop_data_t* _data)
 {
-	const struct linop_cdf97_s* data = _data;
+	const struct linop_cdf97_s* data = CAST_DOWN(linop_cdf97_s, _data);
+
 	free((void*)data->dims);
+
 	free((void*)data);
 }
 
@@ -898,35 +940,51 @@ static void linop_cdf97_free(const void* _data)
  */
 struct linop_s* linop_cdf97_create(int N, const long dims[N], unsigned int flags)
 {
-	struct linop_cdf97_s* data = xmalloc(sizeof(struct linop_cdf97_s));
+	PTR_ALLOC(struct linop_cdf97_s, data);
+	SET_TYPEID(linop_cdf97_s, data);
 
-	long* ndims = xmalloc(N * sizeof(long));
-	md_copy_dims(N, ndims, dims);
+	PTR_ALLOC(long[N], ndims);
+	md_copy_dims(N, *ndims, dims);
 
 	data->N = N;
-	data->dims = ndims;
+	data->dims = *ndims;
 	data->flags = flags;
 
-	return linop_create(N, dims, N, dims, data, linop_cdf97_apply, linop_cdf97_adjoint, linop_cdf97_normal, NULL, linop_cdf97_free);
+	return linop_create(N, dims, N, dims, CAST_UP(PTR_PASS(data)), linop_cdf97_apply, linop_cdf97_adjoint, linop_cdf97_normal, NULL, linop_cdf97_free);
 }
 
 
-static void linop_conv_forward(const void* _data, complex float* out, const complex float* in)
+
+struct conv_data_s {
+
+	INTERFACE(linop_data_t);
+
+	struct conv_plan* plan;
+};
+
+DEF_TYPEID(conv_data_s);
+
+static void linop_conv_forward(const linop_data_t* _data, complex float* out, const complex float* in)
 {
-	struct conv_plan* plan = (void*)_data;
-	conv_exec(plan, out, in);
+	struct conv_data_s* data = CAST_DOWN(conv_data_s, _data);
+
+	conv_exec(data->plan, out, in);
 }
 
-static void linop_conv_adjoint(const void* _data, complex float* out, const complex float* in)
+static void linop_conv_adjoint(const linop_data_t* _data, complex float* out, const complex float* in)
 {
-	struct conv_plan* plan = (void*)_data;
-	conv_adjoint(plan, out, in);
+	struct conv_data_s* data = CAST_DOWN(conv_data_s, _data);
+
+	conv_adjoint(data->plan, out, in);
 }
 
-static void linop_conv_free(const void* _data)
+static void linop_conv_free(const linop_data_t* _data)
 {
-	struct conv_plan* plan = (void*)_data;
-	conv_free(plan);
+	struct conv_data_s* data = CAST_DOWN(conv_data_s, _data);
+
+	conv_free(data->plan);
+
+	free(data);
 }
 
 
@@ -945,9 +1003,12 @@ static void linop_conv_free(const void* _data)
 struct linop_s* linop_conv_create(int N, unsigned int flags, enum conv_type ctype, enum conv_mode cmode, const long odims[N],
                 const long idims[N], const long kdims[N], const complex float* krn)
 {
-	struct conv_plan* plan = conv_plan(N, flags, ctype, cmode, odims, idims, kdims, krn);
+	PTR_ALLOC(struct conv_data_s, data);
+	SET_TYPEID(conv_data_s, data);
 
-	return linop_create(N, odims, N, idims, plan, linop_conv_forward, linop_conv_adjoint, NULL, NULL, linop_conv_free);
+	data->plan = conv_plan(N, flags, ctype, cmode, odims, idims, kdims, krn);
+
+	return linop_create(N, odims, N, idims, CAST_UP(PTR_PASS(data)), linop_conv_forward, linop_conv_adjoint, NULL, NULL, linop_conv_free);
 }
 
 
