@@ -14,6 +14,7 @@
 
 #include "num/multind.h"
 #include "num/flpmath.h"
+#include "num/init.h"
 
 #include "misc/misc.h"
 #include "misc/mmio.h"
@@ -28,8 +29,8 @@
 
 
 
-static const char* usage_str = "flags <input> <output>";
-static const char* help_str = "Normalize along selected dimensions.\n";
+static const char usage_str[] = "flags <input> <output>";
+static const char help_str[] = "Normalize along selected dimensions.\n";
 
 
 
@@ -39,6 +40,8 @@ int main_normalize(int argc, char* argv[])
 
 	l1 = mini_cmdline_bool(argc, argv, 'b', 3, usage_str, help_str);
 
+	num_init();
+
 	int N = DIMS;
 	long dims[N];
 	complex float* data = load_cfl(argv[2], N, dims);
