@@ -1,19 +1,8 @@
 
-
-
-#ifdef __cplusplus
-extern "C" {
-#ifndef __VLA
-#define __VLA(x) 
-#endif
-#else
-#ifndef __VLA
-#define __VLA(x) static x
-#endif
-#endif
-
 #include <complex.h>
 
+#include "misc/cppwrap.h"
+
 extern float median_float(int N, float ar[N]);
 extern complex float median_complex_float(int N, complex float ar[N]);
 
@@ -25,7 +14,5 @@ extern void linear_phase(unsigned int N, const long dims[__VLA(N)], const float
 extern void centered_gradient(unsigned int N, const long dims[__VLA(N)], const _Complex float grad[__VLA(N)], _Complex float* out);
 extern void klaplace(unsigned int N, const long dims[__VLA(N)], unsigned int flags, _Complex float* out);
 
-#ifdef __cplusplus
-}
-#endif
+#include "misc/cppwrap.h"
 
