@@ -1,10 +1,11 @@
 /* Copyright 2013-2015. The Regents of the University of California.
  * Copyright 2014. Joseph Y Cheng.
+ * Copyright 2016. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors: 
- * 2012-2015	Martin Uecker <uecker@eecs.berkeley.edu>
+ * 2012-2016	Martin Uecker <martin.uecker@med.uni-goettingen.de>
  * 2014 	Joseph Y Cheng <jycheng@stanford.edu>
  * 2015		Jonathan Tamir <jtamir@eecs.berkeley.edu>
  *
@@ -16,7 +17,6 @@
 
 #ifdef USE_CUDA
 
-#include <stdio.h>
 #include <stdbool.h>
 #include <assert.h>
 
@@ -29,6 +29,7 @@
 #include "num/vecops.h"
 #include "num/gpuops.h"
 #include "num/gpukrnls.h"
+#include "num/mem.h"
 
 #include "misc/misc.h"
 #include "misc/debug.h"
@@ -155,145 +156,17 @@ static void cuda_float_copy(long size, float* dst, const float* src)
 }
 
 
-struct cuda_mem_s {
-
-	const void* ptr;
-	size_t len;
-	bool device;
-	bool free;
-	int device_id;
-	int thread_id;
-	struct cuda_mem_s* next;
-};
-
-static struct cuda_mem_s* cuda_mem_list = NULL;
-
-
-static bool inside_p(const struct cuda_mem_s* rptr, const void* ptr)
+static void cuda_free_wrapper(const void* ptr)
 {
-	return (ptr >= rptr->ptr) && (ptr < rptr->ptr + rptr->len);
-}
-
-static struct cuda_mem_s* search(const void* ptr, bool remove)
-{
-	struct cuda_mem_s* rptr = NULL;
-
-	#pragma omp critical
-	{
-		struct cuda_mem_s** nptr = &cuda_mem_list;
-
-		while (true) {
-
-			rptr = *nptr;
-
-			if (NULL == rptr)
-				break;
-
-			if (inside_p(rptr, ptr)) {
-
-				if (remove)
-					*nptr = rptr->next;
-
-				break;
-			}
-
-			nptr = &(rptr->next);
-		}
-	}
-
-	return rptr;
-}
-
-static bool free_check_p(const struct cuda_mem_s* rptr, size_t size, int tid)
-{
-	return (rptr->free && (rptr->device_id == last_init) && (rptr->len >= size)
-			&& ((-1 == tid) || (rptr->thread_id == tid)));
-}
-
-static struct cuda_mem_s** find_free_unsafe(size_t size, int tid)
-{
-	struct cuda_mem_s* rptr = NULL;
-	struct cuda_mem_s** nptr = &cuda_mem_list;
-
-	while (true) {
-
-		rptr = *nptr;
-
-		if (NULL == rptr)
-			break;
-
-		if (free_check_p(rptr, size, tid))
-			break;
-
-		nptr = &(rptr->next);
-	}
-
-	return nptr;
-}
-
-static struct cuda_mem_s* find_free(size_t size)
-{
-	struct cuda_mem_s* rptr = NULL;
-
-	#pragma omp critical
-	{
-		rptr = *find_free_unsafe(size, -1);
-
-		if (NULL != rptr)
-			rptr->free = false;
-	}
-
-	return rptr;
-}
-
-static void insert(const void* ptr, size_t len, bool device)
-{
-	struct cuda_mem_s* nptr = xmalloc(sizeof(struct cuda_mem_s));
-	nptr->ptr = ptr;
-	nptr->len = len;
-	nptr->device = device;
-	nptr->device_id = last_init;
-	nptr->thread_id = omp_get_thread_num();
-	nptr->free = false;
-
-	#pragma omp critical
-	{
-		nptr->next = cuda_mem_list;
-		cuda_mem_list = nptr;
-	}
+	CUDA_ERROR(cudaFree((void*)ptr));
 }
 
 void cuda_memcache_clear(void)
 {
-	struct cuda_mem_s* nptr = NULL;
-
 	if (!cuda_memcache)
 		return;
 
-	do {
-		#pragma omp critical
-		{
-			struct cuda_mem_s** rptr = find_free_unsafe(0, omp_get_thread_num());
-			nptr = *rptr;
-
-			// remove from list
-
-			if (NULL != nptr)
-				*rptr = nptr->next;
-		}
-
-		if (NULL != nptr) {
-
-			assert(nptr->device);
-
-			debug_printf(DP_DEBUG3, "Freeing %ld bytes. (DID: %d TID: %d)\n\n",
-					nptr->len, nptr->device_id, nptr->thread_id);
-
-			cudaFree((void*)nptr->ptr);
-			free(nptr);
-		}
-
-	} while (NULL != nptr);
+	memcache_clear(last_init, cuda_free_wrapper);
 }
 
 void cuda_exit(void)
@@ -327,22 +200,13 @@ static bool cuda_cuda_ondevice(const void* ptr)
 
 bool cuda_ondevice(const void* ptr)
 {
-	if (NULL == ptr)
-		return false;
-
-	struct cuda_mem_s* p = search(ptr, false);
-	bool r = ((NULL != p) && p->device);
-
-//	assert(r == cuda_cuda_ondevice(ptr));
-
-	return r;
+	return mem_ondevice(ptr);
 }
 
 bool cuda_accessible(const void* ptr)
 {
 #if 1
-	struct cuda_mem_s* p = search(ptr, false);	
-	return (NULL != p);
+	return mem_device_accessible(ptr);
 #else
 	struct cudaPointerAttributes attr;
 	//CUDA_ERROR(cudaPointerGetAttributes(&attr, ptr));
@@ -357,53 +221,26 @@ bool cuda_accessible(const void* ptr)
 
 void cuda_free(void* ptr)
 {
-	struct cuda_mem_s* nptr = search(ptr, !cuda_memcache);
-
-	assert(NULL != nptr);
-	assert(nptr->ptr == ptr);
-	assert(nptr->device);
-
-	if (cuda_memcache) {
-
-		assert(!nptr->free);
-		nptr->free = true;
-
-	} else {
-
-		free(nptr);
-		cudaFree(ptr);
-	}
+	mem_device_free(ptr, cuda_free_wrapper);
 }
 
 
-void* cuda_malloc(long size)
+static void* cuda_malloc_wrapper(size_t size)
 {
-	if (cuda_memcache) {
-
-		struct cuda_mem_s* nptr = find_free(size);
-
-		if (NULL != nptr) {
-
-			assert(nptr->device);
-			assert(!nptr->free);
-
-			nptr->thread_id = omp_get_thread_num();
-
-			return (void*)(nptr->ptr);
-		}
-	}
-
 	void* ptr;
         CUDA_ERROR(cudaMalloc(&ptr, size));
 
-	insert(ptr, size, true);
-
 	return ptr;
 }
 
+void* cuda_malloc(long size)
+{
+	return mem_device_malloc(last_init, size, cuda_malloc_wrapper);
+}
 
 
 
+#if 0
 void* cuda_hostalloc(long N)
 {
 	void* ptr;
@@ -423,6 +260,7 @@ void cuda_hostfree(void* ptr)
 
 	cudaFreeHost(ptr);
 }
+#endif
 
 static float* cuda_float_malloc(long size)
 {
@@ -477,19 +315,11 @@ static void cuda_swap(long size, float* a, float* b)
 
 const struct vec_ops gpu_ops = {
 
-	.allocate = cuda_float_malloc,
-	.del = cuda_float_free,
-	.clear = cuda_float_clear,
-	.copy = cuda_float_copy,
 	.float2double = cuda_float2double,
 	.double2float = cuda_double2float,
 	.dot = cuda_sdot,
-	.norm = cuda_norm,
 	.asum = cuda_asum,
 	.zl1norm = NULL,
-	.axpy = cuda_saxpy,
-	.xpay = cuda_xpay,
-	.smul = cuda_smul,
 
 	.add = cuda_add,
 	.sub = cuda_sub,
@@ -498,6 +328,8 @@ const struct vec_ops gpu_ops = {
 	.fmac = cuda_fmac,
 	.fmac2 = cuda_fmac2,
 
+	.axpy = cuda_saxpy,
+
 	.pow = cuda_pow,
 	.sqrt = cuda_sqrt,
 
@@ -529,8 +361,6 @@ const struct vec_ops gpu_ops = {
 	.zsoftthresh_half = cuda_zsoftthresh_half,
 	.softthresh = cuda_softthresh,
 	.softthresh_half = cuda_softthresh_half,
-
-	.swap = cuda_swap,
 };
 
 
