@@ -1,9 +1,10 @@
 /* Copyright 2013-2015 The Regents of the University of California.
- * All rights reserved. Use of this source code is governed by 
+ * Copyright 2016. Martin Uecker.
+ * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors:
- * 2012-2015 Martin Uecker <uecker@eecs.berkeley.edu>
+ * 2012-2016 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  * 2013      Frank Ong <frankong@berkeley.edu>
  *
  * Generic operations on multi-dimensional arrays. Most functions
@@ -22,6 +23,8 @@
  *
  */
 
+#define _GNU_SOURCE
+
 #include <string.h>
 #include <assert.h>
 #include <stdbool.h>
@@ -29,6 +32,7 @@
 #include <strings.h>
 
 #include "misc/misc.h"
+#include "misc/types.h"
 #include "misc/debug.h"
 
 #include "num/optimize.h"
@@ -57,7 +61,7 @@ void md_nary(unsigned int C, unsigned int D, const long dim[D], const long* str[
 	for (long i = 0; i < dim[D - 1]; i++) {
 
 		void* moving_ptr[C];
-	
+
 		for (unsigned int j = 0; j < C; j++)
 			moving_ptr[j] = ptr[j] + i * str[j][D - 1];
 
@@ -73,7 +77,7 @@ void md_nary(unsigned int C, unsigned int D, const long dim[D], const long* str[
  * This functions tries to parallelize over the dimensions indicated
  * with flags.
  */
-void md_parallel_nary(unsigned int C, unsigned int D, const long dim[D], unsigned int flags, const long* str[C], void* ptr[C], void* data, md_nary_fun_t fun)
+void md_parallel_nary(unsigned int C, unsigned int D, const long dim[D], unsigned long flags, const long* str[C], void* ptr[C], void* data, md_nary_fun_t fun)
 {
 	if (0 == flags) {
 
@@ -81,7 +85,7 @@ void md_parallel_nary(unsigned int C, unsigned int D, const long dim[D], unsigne
 		return;
 	}
 
-	int b = ffs(flags & -flags) - 1;
+	int b = ffsl(flags & -flags) - 1;
 	assert(MD_IS_SET(flags, b));
 
 	flags = MD_CLEAR(flags, b);
@@ -107,6 +111,46 @@ void md_parallel_nary(unsigned int C, unsigned int D, const long dim[D], unsigne
 
 
 
+static void md_parallel_loop_r(unsigned int D, unsigned int N, const long dim[static N], unsigned int flags, const long pos[static N], void* data, md_loop_fun_t fun)
+{
+	if (0 == D) {
+
+		fun(data, pos);
+		return;
+	}
+
+	D--;
+
+	// we need to make a copy because firstprivate needs to see
+	// an array instead of a pointer
+	long pos_copy[N];
+	for (unsigned int i = 0; i < N; i++)
+		pos_copy[i] = pos[i];
+
+	#pragma omp parallel for firstprivate(pos_copy) if ((1 < dim[D]) && (flags & (1 << D)))
+	for (int i = 0; i < dim[D]; i++) {
+
+		pos_copy[D] = i;
+
+		md_parallel_loop_r(D, N, dim, flags, pos_copy, data, fun);
+	}
+}
+
+/**
+ * Generic function which loops over all dimensions and calls a given
+ * function passing the current indices as argument.
+ *
+ * Runs fun(data, position) for all position in dim
+ *
+ */
+void md_parallel_loop(unsigned int D, const long dim[static D], unsigned long flags, void* data, md_loop_fun_t fun)
+{
+	long pos[D];
+	md_parallel_loop_r(D, D, dim, flags, pos, data, fun);
+}
+
+
+
 static void md_loop_r(unsigned int D, const long dim[D], long pos[D], void* data, md_loop_fun_t fun)
 {
 	if (0 == D) {
@@ -117,7 +161,7 @@ static void md_loop_r(unsigned int D, const long dim[D], long pos[D], void* data
 
 	D--;
 
-	for (pos[D] = 0; pos[D] < dim[D]; pos[D]++) 
+	for (pos[D] = 0; pos[D] < dim[D]; pos[D]++)
 		md_loop_r(D, dim, pos, data, fun);
 }
 
@@ -139,7 +183,7 @@ void md_loop(unsigned int D, const long dim[D], void* data, md_loop_fun_t fun)
 /**
  * Computes the next position. Returns true until last index.
  */
-bool md_next(unsigned int D, const long dims[D], unsigned int flags, long pos[D])
+bool md_next(unsigned int D, const long dims[D], unsigned long flags, long pos[D])
 {
 	if (0 == D--)
 		return false;
@@ -224,7 +268,7 @@ unsigned int md_calc_blockdim(unsigned int D, const long dim[D], const long str[
 	}
 
 	return i;
-}	
+}
 
 
 
@@ -241,7 +285,7 @@ unsigned int md_calc_blockdim(unsigned int D, const long dim[D], const long str[
 void md_select_dims(unsigned int D, unsigned long flags, long odims[D], const long idims[D])
 {
 	md_copy_dims(D, odims, idims);
-	
+
 	for (unsigned int i = 0; i < D; i++)
 		if (!MD_IS_SET(flags, i))
 			odims[i] = 1;
@@ -306,6 +350,7 @@ bool md_check_dimensions(unsigned int N, const long dims[N], unsigned int flags)
 {
 	long d[N];
 	md_select_dims(N, ~flags, d, dims);
+
 	return (1 != md_calc_size(N, d));
 }
 
@@ -351,12 +396,22 @@ bool md_check_compat(unsigned int D, unsigned long flags, const long dim1[D], co
 
 	if ((dim1[D] == dim2[D]) || (MD_IS_SET(flags, D) && ((1 == dim1[D]) || (1 == dim2[D]))))
 		return md_check_compat(D, flags, dim1, dim2);
-		
+
 	return false;
 }
 
 
 
+void md_merge_dims(unsigned int N, long out_dims[N], const long dims1[N], const long dims2[N])
+{
+	assert(md_check_compat(N, ~0, dims1, dims2));
+
+	for (unsigned int i = 0; i < N; i++)
+		out_dims[i] = (1 == dims1[i]) ? dims2[i] : dims1[i];
+}
+
+
+
 /**
  * dim1 must be bounded by dim2 where a bit is set
  */
@@ -400,17 +455,19 @@ struct data_s {
 #endif
 };
 
-static void nary_clear(void* _data, void* ptr[])
+static void nary_clear(struct nary_opt_data_s* opt_data, void* ptr[])
 {
-	struct data_s* data = (struct data_s*)_data;
+	struct data_s* data = opt_data->data_ptr;
+	size_t size = data->size * opt_data->size;
+
 #ifdef  USE_CUDA
 	if (data->use_gpu) {
 
-		cuda_clear(data->size, ptr[0]);
+		cuda_clear(size, ptr[0]);
 		return;
 	}
 #endif
-	memset(ptr[0], 0, data->size);	
+	memset(ptr[0], 0, size);
 }
 
 /**
@@ -420,14 +477,13 @@ static void nary_clear(void* _data, void* ptr[])
  */
 void md_clear2(unsigned int D, const long dim[D], const long str[D], void* ptr, size_t size)
 {
-	int skip = md_calc_blockdim(D, dim, str, size);
-//	printf("CLEAR skip %d\n", skip);
-#ifdef  USE_CUDA
-	struct data_s data = { md_calc_size(skip, dim) * size, cuda_ondevice(ptr) };
+	const long (*nstr[1])[D] = { (const long (*)[D])str };
+#ifdef	USE_CUDA
+	struct data_s data = { size, cuda_ondevice(ptr) };
 #else
-	struct data_s data = { md_calc_size(skip, dim) * size };
+	struct data_s data = { size };
 #endif
-	md_nary(1, D - skip, dim + skip, (const long*[1]){ str + skip }, (void*[1]){ ptr }, (void*)&data, &nary_clear);
+	optimized_nop(1, MD_BIT(0), D, dim, nstr, (void*[1]){ ptr }, (size_t[1]){ size }, nary_clear, &data);
 }
 
 
@@ -441,7 +497,7 @@ void md_clear2(unsigned int D, const long dim[D], const long str[D], void* ptr,
  * @param dim array of dimensions
  * @param size of a single element
  */
-void md_calc_strides(unsigned int D, long str[D], const long dim[D], size_t size)
+long* md_calc_strides(unsigned int D, long str[D], const long dim[D], size_t size)
 {
 	long old = size;
 
@@ -450,6 +506,8 @@ void md_calc_strides(unsigned int D, long str[D], const long dim[D], size_t size
 		str[i] = (1 == dim[i]) ? 0 : old;
 		old *= dim[i];
 	}
+
+	return str;
 }
 
 
@@ -466,9 +524,7 @@ void md_calc_strides(unsigned int D, long str[D], const long dim[D], size_t size
  */
 void md_clear(unsigned int D, const long dim[D], void* ptr, size_t size)
 {
-	long str[D];
-	md_calc_strides(D, str, dim, size);
-	md_clear2(D, dim, str, ptr, size);
+	md_clear2(D, dim, MD_STRIDES(D, dim, size), ptr, size);
 }
 
 
@@ -490,19 +546,20 @@ static void nary_strided_copy(void* _data, void* ptr[])
 }
 #endif
 
-static void nary_copy(void* _data, void* ptr[])
+static void nary_copy(struct nary_opt_data_s* opt_data, void* ptr[])
 {
-	struct data_s* data = (struct data_s*)_data;
+	struct data_s* data = opt_data->data_ptr;
+	size_t size = data->size * opt_data->size;
+
 #ifdef  USE_CUDA
 	if (data->use_gpu) {
 
-//		printf("CUDA copy %ld\n", data->size);
-
-		cuda_memcpy(data->size, ptr[0], ptr[1]);
+		cuda_memcpy(size, ptr[0], ptr[1]);
 		return;
 	}
 #endif
-	memcpy(ptr[0], ptr[1], data->size);	
+
+	memcpy(ptr[0], ptr[1], size);
 }
 
 /**
@@ -518,10 +575,15 @@ void md_copy2(unsigned int D, const long dim[D], const long ostr[D], void* optr,
 	extern void fft2(unsigned int D, const long dim[D], unsigned int flags, 
 			const long ostr[D], void* optr, const long istr[D], const void* iptr);
 
-	if (sizeof(_Complex float) == size)
+	if (sizeof(complex float) == size)
 		fft2(D, dim, 0, ostr, optr, istr, iptr);
 #endif
 
+#ifndef	USE_CUDA
+	struct data_s data = { size };
+#else
+	struct data_s data = { size, cuda_ondevice(optr) || cuda_ondevice(iptr) };
+#if 1
 	long tostr[D];
 	long tistr[D];
 	long tdims[D];
@@ -532,34 +594,33 @@ void md_copy2(unsigned int D, const long dim[D], const long ostr[D], void* optr,
 
 	long (*nstr2[2])[D] = { &tostr, &tistr };
 	int ND = optimize_dims(2, D, tdims, nstr2);
+
 	size_t sizes[2] = { size, size };
-	int skip = min_blockdim(2, ND, tdims, nstr2, sizes); 
+	int skip = min_blockdim(2, ND, tdims, nstr2, sizes);
 
-	const long* nstr[2] = { *nstr2[0] + skip, *nstr2[1] + skip };
 
-	void* nptr[2] = { optr, (void*)iptr };
-#ifdef  USE_CUDA
-	struct data_s data = { md_calc_size(skip, tdims) * size, (cuda_ondevice(optr) || cuda_ondevice(iptr)) };
-
-#if 1
 	if (data.use_gpu && (ND - skip == 1)) { 
 		// FIXME: the test was > 0 which would optimize transpose
 		// but failes in the second cuda_memcpy_strided call
 		// probably because of alignment restrictions
+		const long* nstr[2] = { *nstr2[0] + skip, *nstr2[1] + skip };
+
+		void* nptr[2] = { optr, (void*)iptr };
 
 		long sizes[2] = { md_calc_size(skip, tdims) * size, tdims[skip] };
 		struct strided_copy_s data = { { sizes[0], sizes[1] } , (*nstr2[0])[skip], (*nstr2[1])[skip] };
 
 		skip++;
-		md_nary(2, ND - skip, tdims + skip , nstr, nptr, (void*)&data, &nary_strided_copy);
+
+		md_nary(2, ND - skip, tdims + skip , nstr, nptr, &data, &nary_strided_copy);
 		return;
 	}
 #endif
-#else
-	struct data_s data = { md_calc_size(skip, tdims) * size };
 #endif
 
-	md_nary(2, ND - skip, tdims + skip, nstr, nptr, (void*)&data, &nary_copy);
+	const long (*nstr[2])[D] = { (const long (*)[D])ostr, (const long (*)[D])istr };
+
+	optimized_nop(2, MD_BIT(0), D, dim, nstr, (void*[2]){ optr, (void*)iptr }, (size_t[2]){ size, size }, nary_copy, &data);
 }
 
 
@@ -573,6 +634,7 @@ void md_copy(unsigned int D, const long dim[D], void* optr, const void* iptr, si
 {
 	long str[D];
 	md_calc_strides(D, str, dim, size);
+
 	md_copy2(D, dim, str, optr, str, iptr, size);
 }
 
@@ -586,7 +648,6 @@ static void* gpu_constant(const void* vp, size_t size)
 }
 #endif
 
-
 /**
  * Fill array with value pointed by pointer (with strides)
  *
@@ -598,13 +659,17 @@ void md_fill2(unsigned int D, const long dim[D], const long str[D], void* ptr, c
 	if (cuda_ondevice(ptr) && (!cuda_ondevice(iptr))) {
 
 		void* giptr = gpu_constant(iptr, size);
+
 		md_fill2(D, dim, str, ptr, giptr, size);
+
 		md_free(giptr);
 		return;
 	}
 #endif
+
 	long istr[D];
 	md_singleton_strides(D, istr);
+
 	md_copy2(D, dim, str, ptr, istr, iptr, size);
 }
 
@@ -617,9 +682,7 @@ void md_fill2(unsigned int D, const long dim[D], const long str[D], void* ptr, c
  */
 void md_fill(unsigned int D, const long dim[D], void* ptr, const void* iptr, size_t size)
 {
-	long str[D];
-	md_calc_strides(D, str, dim, size);
-	md_fill2(D, dim, str, ptr, iptr, size);
+	md_fill2(D, dim, MD_STRIDES(D, dim, size), ptr, iptr, size);
 }
 
 
@@ -630,12 +693,14 @@ struct swap_s {
 	size_t size;
 };
 
-static void nary_swap(void* _data, void* ptr[])
+static void nary_swap(struct nary_opt_data_s* opt_data, void* ptr[])
 {
-	const struct swap_s* data = _data;
-	size_t size = data->size;
+	const struct swap_s* data = opt_data->data_ptr;
+	size_t size = data->size * opt_data->size;
 	unsigned int M = data->M;
+
 	char* tmp = (size < 32) ? alloca(size) : xmalloc(size);
+
 #ifdef  USE_CUDA
 	assert(!cuda_ondevice(ptr[0]));
 	assert(!cuda_ondevice(ptr[1]));
@@ -648,26 +713,25 @@ static void nary_swap(void* _data, void* ptr[])
 	memcpy(ptr[M - 1], tmp, size);
 
 	if (size >= 32)
-		free(tmp);
+		xfree(tmp);
 }
 
 /**
  * Swap values between a number of arrays (with strides)
  */
-void md_circular_swap2(unsigned M, unsigned int D, const long dims[D], const long* strs[M], void* ptr[M], size_t size)
-//void md_circular_swap2(unsigned M, unsigned int D, const long dims[D], const long strs[M][D], void* ptr[M], size_t size)
+void md_circular_swap2(unsigned int M, unsigned int D, const long dims[D], const long* strs[M], void* ptr[M], size_t size)
 {
-	unsigned int skip = md_calc_blockdim(D, dims, strs[0], size);
-
-	for (unsigned int i = 1; i < M; i++)
-		skip = MIN(skip, md_calc_blockdim(D, dims, strs[i], size));
-
-	const long* nstr[M];
+	size_t sizes[M];
 	for (unsigned int i = 0; i < M; i++)
-		nstr[i] = strs[i] + skip;
+		sizes[i] = size;
 
-	struct swap_s data = { M, md_calc_size(skip, dims) * size };
-	md_nary(M, D - skip, dims + skip, nstr, ptr, (void*)&data, &nary_swap);
+	struct swap_s data = { M, size };
+
+	const long (*nstrs[M])[D];
+	for (unsigned int i = 0; i < M; i++)
+		nstrs[i] = (const long (*)[D])strs[i];
+
+	optimized_nop(M, (1 << M) - 1, D, dims, nstrs, ptr, sizes, nary_swap, &data);
 }
 
 
@@ -683,6 +747,8 @@ void md_circular_swap(unsigned M, unsigned int D, const long dims[D], void* ptr[
 
 	const long* strp[M];
 
+	strp[0] = strs[0];
+
 	for (unsigned int i = 1; i < M; i++) {
 
 		md_copy_strides(D, strs[i], strs[0]);
@@ -715,6 +781,7 @@ void md_swap(unsigned int D, const long dim[D], void* optr, void* iptr, size_t s
 {
 	long str[D];
 	md_calc_strides(D, str, dim, size);
+
 	md_swap2(D, dim, str, optr, str, iptr, size);
 }
 
@@ -747,19 +814,16 @@ void md_move_block2(unsigned int D, const long dim[D], const long opos[D], const
  */
 void md_move_block(unsigned int D, const long dim[D], const long opos[D], const long odim[D], void* optr, const long ipos[D], const long idim[D], const void* iptr, size_t size)
 {
-	long istr[D];
-	long ostr[D];
-	md_calc_strides(D, istr, idim, size);
-	md_calc_strides(D, ostr, odim, size);
-
-	md_move_block2(D, dim, opos, odim, ostr, optr, ipos, idim, istr, iptr, size);
+	md_move_block2(D, dim,
+			opos, odim, MD_STRIDES(D, odim, size), optr,
+			ipos, idim, MD_STRIDES(D, idim, size), iptr, size);
 }
 
 
 /**
  * Copy a block from an array to another array (with strides)
  *
- * Block dimensions are min( idim , odim )
+ * Block dimensions are min(idim , odim)
  *
  * if idim[d] > odim[d], then optr[i] = iptr[pos + i] for 0 <= i < odim[d]
  *
@@ -795,7 +859,7 @@ void md_copy_block2(unsigned int D, const long pos[D], const long odim[D], const
 /**
  * Copy a block from an array to another array (without strides)
  *
- * Block dimensions are min( idim , odim )
+ * Block dimensions are min(idim , odim)
  *
  * if idim[d] > odim[d], then optr[i] = iptr[pos + i] for 0 <= i < odim[d]
  *
@@ -804,12 +868,9 @@ void md_copy_block2(unsigned int D, const long pos[D], const long odim[D], const
  */
 void md_copy_block(unsigned int D, const long pos[D], const long odim[D], void* optr, const long idim[D], const void* iptr, size_t size)
 {
-	long istr[D];
-	long ostr[D];
-	md_calc_strides(D, istr, idim, size);
-	md_calc_strides(D, ostr, odim, size);
-
-	md_copy_block2(D, pos, odim, ostr, optr, idim, istr, iptr, size);
+	md_copy_block2(D, pos,
+			odim, MD_STRIDES(D, odim, size), optr,
+			idim, MD_STRIDES(D, idim, size), iptr, size);
 }
 
 
@@ -824,6 +885,7 @@ void md_resize(unsigned int D, const long odim[D], void* optr, const long idim[D
 {
 	long pos[D];
 	memset(pos, 0, D * sizeof(long));
+
 	md_clear(D, odim, optr, size);
 	md_copy_block(D, pos, odim, optr, idim, iptr, size);
 }
@@ -853,13 +915,14 @@ void md_resize_center(unsigned int D, const long odim[D], void* optr, const long
 /**
  * Extract slice from array specified by flags (with strides)
  *
- * optr = iptr( pos[0] , : , pos[2] , : , : )
+ * optr = iptr(pos[0], :, pos[2], :, :)
  *
  */
 void md_slice2(unsigned int D, unsigned long flags, const long pos[D], const long dim[D], const long ostr[D], void* optr, const long istr[D], const void* iptr, size_t size)
 {
 	long odim[D];
 	md_select_dims(D, ~flags, odim, dim);
+
 	md_copy_block2(D, pos, odim, ostr, optr, dim, istr, iptr, size);
 }
 
@@ -868,21 +931,17 @@ void md_slice2(unsigned int D, unsigned long flags, const long pos[D], const lon
 /**
  * Extract slice from array specified by flags (with strides)
  *
- * optr = iptr( pos[0] , : , pos[2] , : , : )
+ * optr = iptr(pos[0], :, pos[2], :, :)
  *
  */
 void md_slice(unsigned int D, unsigned long flags, const long pos[D], const long dim[D], void* optr, const void* iptr, size_t size)
 {
-	long ostr[D];
-	long istr[D];
-
 	long odim[D];
 	md_select_dims(D, ~flags, odim, dim);
 
-	md_calc_strides(D, istr, dim, size);
-	md_calc_strides(D, ostr, odim, size);
-
-	md_slice2(D, flags, pos, dim, ostr, optr, istr, iptr, size);
+	md_slice2(D, flags, pos, dim,
+			MD_STRIDES(D, odim, size), optr,
+			MD_STRIDES(D, dim, size), iptr, size);
 }
 
 
@@ -890,7 +949,7 @@ void md_slice(unsigned int D, unsigned long flags, const long pos[D], const long
 /**
  * Permute array (with strides)
  *
- * optr[ order[i] ] = iptr[i]
+ * optr[order[i]] = iptr[i]
  *
  */
 void md_permute2(unsigned int D, const unsigned int order[D], const long odims[D], const long ostr[D], void* optr, const long idims[D], const long istr[D], const void* iptr, size_t size)
@@ -902,6 +961,7 @@ void md_permute2(unsigned int D, const unsigned int order[D], const long odims[D
 
 		assert(order[i] < D);
 		assert(odims[i] == idims[order[i]]);
+
 		flags = MD_SET(flags, order[i]);
 
 		ostr2[order[i]] = ostr[i];
@@ -917,18 +977,14 @@ void md_permute2(unsigned int D, const unsigned int order[D], const long odims[D
 /**
  * Permute array (without strides)
  *
- * optr[ order[i] ] = iptr[i]
+ * optr[order[i]] = iptr[i]
  *
  */
 void md_permute(unsigned int D, const unsigned int order[D], const long odims[D], void* optr, const long idims[D], const void* iptr, size_t size)
 {
-	long ostr[D];
-	long istr[D];
-
-	md_calc_strides(D, istr, idims, size);
-	md_calc_strides(D, ostr, odims, size);
-
-	md_permute2(D, order, odims, ostr, optr, idims, istr, iptr, size);
+	md_permute2(D, order,
+			odims, MD_STRIDES(D, odims, size), optr,
+			idims, MD_STRIDES(D, idims, size), iptr, size);
 }
 
 
@@ -967,6 +1023,7 @@ void md_transpose_dims(unsigned int D, unsigned int dim1, unsigned int dim2, lon
 {
 	unsigned int order[D];
 	md_transpose_order(D, order, dim1, dim2);
+
 	md_permute_dims(D, order, odims, idims);
 }
 
@@ -975,9 +1032,9 @@ void md_transpose_dims(unsigned int D, unsigned int dim1, unsigned int dim2, lon
 /**
  * Tranpose array (with strides)
  *
- * optr[ dim2 ] = iptr[ dim1]
+ * optr[dim2] = iptr[dim1]
  *
- * optr[ dim1 ] = iptr[ dim2]
+ * optr[dim1] = iptr[dim2]
  *
  */
 void md_transpose2(unsigned int D, unsigned int dim1, unsigned int dim2, const long odims[D], const long ostr[D], void* optr, const long idims[D], const long istr[D], const void* iptr, size_t size)
@@ -991,7 +1048,8 @@ void md_transpose2(unsigned int D, unsigned int dim1, unsigned int dim2, const l
 
 	unsigned int order[D];
 	md_transpose_order(D, order, dim1, dim2);
-	md_permute2(D, order, odims, ostr, optr, idims, istr, iptr, size);	
+
+	md_permute2(D, order, odims, ostr, optr, idims, istr, iptr, size);
 }
 
 
@@ -999,20 +1057,16 @@ void md_transpose2(unsigned int D, unsigned int dim1, unsigned int dim2, const l
 /**
  * Tranpose array (without strides)
  *
- * optr[ dim2 ] = iptr[ dim1]
+ * optr[dim2] = iptr[dim1]
  *
- * optr[ dim1 ] = iptr[ dim2]
+ * optr[dim1] = iptr[dim2]
  *
  */
 void md_transpose(unsigned int D, unsigned int dim1, unsigned int dim2, const long odims[D], void* optr, const long idims[D], const void* iptr, size_t size)
 {
-	long ostr[D];
-	long istr[D];
-
-	md_calc_strides(D, istr, idims, size);
-	md_calc_strides(D, ostr, odims, size);
-
-	md_transpose2(D, dim1, dim2, odims, ostr, optr, idims, istr, iptr, size);
+	md_transpose2(D, dim1, dim2,
+			odims, MD_STRIDES(D, odims, size), optr,
+			idims, MD_STRIDES(D, idims, size), iptr, size);
 }
 
 
@@ -1077,6 +1131,7 @@ void md_swap_flip(unsigned int D, const long dims[D], unsigned long flags, void*
 {
 	long strs[D];
 	md_calc_strides(D, strs, dims, size);
+
 	md_swap_flip2(D, dims, flags, strs, optr, strs, iptr, size);
 }
 
@@ -1100,13 +1155,14 @@ static void md_flip_inpl2(unsigned int D, const long dims[D], unsigned long flag
 	dims2[i] = dims[i] / 2;
 
 	long off = str[i] * (0 + (dims[i] + 1) / 2);
+
 	md_swap_flip2(D, dims2, flags, str, ptr, str, ptr + off, size);
 }
 
 /**
  * Flip array (with strides)
  *
- * optr[ dims[D] - 1 - i ] = iptr[ i ]
+ * optr[dims[D] - 1 - i] = iptr[i]
  *
  */
 void md_flip2(unsigned int D, const long dims[D], unsigned long flags, const long ostr[D], void* optr, const long istr[D], const void* iptr, size_t size)
@@ -1114,6 +1170,7 @@ void md_flip2(unsigned int D, const long dims[D], unsigned long flags, const lon
 	if (optr == iptr) {
 
 		assert(ostr == istr);
+
 		md_flip_inpl2(D, dims, flags, ostr, optr, size);
 		return;
 	}
@@ -1129,7 +1186,7 @@ void md_flip2(unsigned int D, const long dims[D], unsigned long flags, const lon
 
 			ostr2[i] = -ostr[i];
 			off += (dims[i] - 1) * ostr[i];
-		} 
+		}
 	}
 
 	md_copy2(D, dims, ostr2, optr + off, istr, iptr, size);
@@ -1140,17 +1197,59 @@ void md_flip2(unsigned int D, const long dims[D], unsigned long flags, const lon
 /**
  * Flip array (without strides)
  *
- * optr[ dims[D] - 1 - i ] = iptr[ i ]
+ * optr[dims[D] - 1 - i] = iptr[i]
  *
  */
 void md_flip(unsigned int D, const long dims[D], unsigned long flags, void* optr, const void* iptr, size_t size)
 {
 	long str[D];
 	md_calc_strides(D, str, dims, size);
+
 	md_flip2(D, dims, flags, str, optr, str, iptr, size);
 }
 
 
+struct compare_s {
+
+	bool eq;
+	size_t size;
+};
+
+static void nary_cmp(struct nary_opt_data_s* opt_data, void* ptrs[])
+{
+	struct compare_s* data = opt_data->data_ptr;
+	size_t size = data->size * opt_data->size;
+
+	bool eq = (0 == memcmp(ptrs[0], ptrs[1], size));
+
+	#pragma omp critical
+	data->eq &= eq;
+}
+
+bool md_compare2(unsigned int D, const long dims[D], const long str1[D], const void* src1,
+			const long str2[D], const void* src2, size_t size)
+{
+	struct compare_s data = { true, size };
+
+	const long (*nstr[2])[D] = { (const long (*)[D])str1, (const long (*)[D])str2 };
+
+	optimized_nop(2, 0u, D, dims, nstr, (void*[2]){ (void*)src1, (void*)src2 }, (size_t[2]){ size, size }, nary_cmp, &data);
+
+	return data.eq;
+}
+
+
+bool md_compare(unsigned int D, const long dims[D], const void* src1, const void* src2, size_t size)
+{
+	long str[D];
+	md_calc_strides(D, str, dims, size);
+
+	return md_compare2(D, dims, str, src1, str, src2, size);
+}
+
+
+
+
 
 struct septrafo_s {
 
@@ -1173,18 +1272,18 @@ static void md_septrafo_r(unsigned int D, unsigned int R, long dimensions[D], un
 		return;
 
 	md_septrafo_r(D, R, dimensions, flags, strides, ptr, fun, _data);
-                
+
         if (MD_IS_SET(flags, R)) {
 
         	struct septrafo_s data = { dimensions[R], strides[R], _data, fun };
                 void* nptr[1] = { ptr };
                 const long* nstrides[1] = { strides };
-        
+
                 dimensions[R] = 1;      // we made a copy in md_septrafo2
                 //md_nary_parallel(1, D, dimensions, nstrides, nptr, &data, nary_septrafo);
                 md_nary(1, D, dimensions, nstrides, nptr, &data, nary_septrafo);
                 dimensions[R] = data.N;
-        }       
+        }
 }
 
 /**
@@ -1192,7 +1291,7 @@ static void md_septrafo_r(unsigned int D, unsigned int R, long dimensions[D], un
  * 
  */
 void md_septrafo2(unsigned int D, const long dimensions[D], unsigned long flags, const long strides[D], void* ptr, md_trafo_fun_t fun, void* _data)
-{               
+{
         long dimcopy[D];
 	md_copy_dims(D, dimcopy, dimensions);
 
@@ -1203,13 +1302,11 @@ void md_septrafo2(unsigned int D, const long dimensions[D], unsigned long flags,
 
 /**
  * Apply a separable transformation along selected dimensions.
- * 
+ *
  */
-void md_septrafo(unsigned int D, const long dimensions[D], unsigned long flags, void* ptr, size_t size, md_trafo_fun_t fun, void* _data)
+void md_septrafo(unsigned int D, const long dims[D], unsigned long flags, void* ptr, size_t size, md_trafo_fun_t fun, void* _data)
 {
-        long strides[D];
-        md_calc_strides(D, strides, dimensions, size);
-        md_septrafo2(D, dimensions, flags, strides, ptr, fun, _data);    
+        md_septrafo2(D, dims, flags, MD_STRIDES(D, dims, size), ptr, fun, _data);
 }
 
 
@@ -1217,7 +1314,7 @@ void md_septrafo(unsigned int D, const long dimensions[D], unsigned long flags,
 /**
  * Copy diagonals from array specified by flags (with strides)
  *
- * dst( i , i , : , i , : ) = src( i , i , : , i , : )
+ * dst(i, i, :, i, :) = src(i, i, :, i, :)
  *
  */
 void md_copy_diag2(unsigned int D, const long dims[D], unsigned long flags, const long str1[D], void* dst, const long str2[D], const void* src, size_t size)
@@ -1227,7 +1324,7 @@ void md_copy_diag2(unsigned int D, const long dims[D], unsigned long flags, cons
 	long count = -1;
 
 	for (unsigned int i = 0; i < D; i++) {
-	
+
 		if (MD_IS_SET(flags, i)) {
 
 			if (count < 0)
@@ -1235,7 +1332,7 @@ void md_copy_diag2(unsigned int D, const long dims[D], unsigned long flags, cons
 
 			assert(dims[i] == count);
 
-			stride1 += str1[i];	
+			stride1 += str1[i];
 			stride2 += str2[i];
 		}
 	}
@@ -1252,13 +1349,14 @@ void md_copy_diag2(unsigned int D, const long dims[D], unsigned long flags, cons
 /**
  * Copy diagonals from array specified by flags (without strides)
  *
- * dst( i , i , : , i , : ) = src( i , i , : , i , : )
+ * dst(i ,i ,: ,i , :) = src(i ,i ,: ,i ,:)
  *
  */
 void md_copy_diag(unsigned int D, const long dims[D], unsigned long flags, void* dst, const void* src, size_t size)
 {	
 	long str[D];
 	md_calc_strides(D, str, dims, size);
+
 	md_copy_diag2(D, dims, flags, str, dst, str, src, size);
 }
 
@@ -1267,23 +1365,22 @@ void md_copy_diag(unsigned int D, const long dims[D], unsigned long flags, void*
 /**
  * Fill diagonals specified by flags with value (without strides)
  *
- * dst( i , i , : , i , : ) = src[0]
+ * dst(i, i, :, i, :) = src[0]
  *
  */
 void md_fill_diag(unsigned int D, const long dims[D], unsigned long flags, void* dst, const void* src, size_t size)
 {
-	long str1[D];
 	long str2[D];
-
-	md_calc_strides(D, str1, dims, size);
 	md_singleton_strides(D, str2);
-	md_copy_diag2(D, dims, flags, str1, dst, str2, src, size);
-}	
+
+	md_copy_diag2(D, dims, flags, MD_STRIDES(D, dims, size), dst, str2, src, size);
+}
 
 
 
 static void md_circ_shift_inpl2(unsigned int D, const long dims[D], const long center[D], const long strs[D], void* dst, size_t size)
 {
+#if 0
 	long dims1[D];
 	long dims2[D];
 
@@ -1319,12 +1416,33 @@ static void md_circ_shift_inpl2(unsigned int D, const long dims[D], const long c
 	center2[i] = 0;
 
 	md_circ_shift_inpl2(D, dims, center2, strs, dst, size);
+#else
+	// use tmp for now
+	unsigned int i;
+
+	for (i = 0; i < D; i++)
+		if (0 != center[i])
+			break;
+
+	if (i == D)
+		return;
+
+	long tmp_strs[D];
+	md_calc_strides(D, tmp_strs, dims, size);
+
+	void* tmp = md_alloc_sameplace(D, dims, size, dst);
+
+	md_copy2(D, dims, tmp_strs, tmp, strs, dst, size);
+	md_circ_shift2(D, dims, center, strs, dst, tmp_strs, tmp, size);
+
+	md_free(tmp);
+#endif
 }
 
 /**
  * Circularly shift array (with strides)
  *
- * dst[ mod( i + center )  ] = src[i]
+ * dst[mod(i + center)] = src[i]
  *
  */
 void md_circ_shift2(unsigned int D, const long dimensions[D], const long center[D], const long str1[D], void* dst, const long str2[D], const void* src, size_t size)
@@ -1352,6 +1470,7 @@ void md_circ_shift2(unsigned int D, const long dimensions[D], const long center[
 	if (dst == src) {
 
 		assert(str1 == str2);
+
 		md_circ_shift_inpl2(D, dimensions, pos, str1, dst, size);
 		return;
 	}
@@ -1382,13 +1501,14 @@ void md_circ_shift2(unsigned int D, const long dimensions[D], const long center[
 /**
  * Circularly shift array (without strides)
  *
- * dst[ mod( i + center )  ] = src[i]
+ * dst[mod(i + center)] = src[i]
  *
  */
 void md_circ_shift(unsigned int D, const long dimensions[D], const long center[D], void* dst, const void* src, size_t size)
 {
 	long strides[D];
 	md_calc_strides(D, strides, dimensions, size);
+
 	md_circ_shift2(D, dimensions, center, strides, dst, strides, src, size);
 }
 
@@ -1448,17 +1568,12 @@ void md_circ_ext2(unsigned int D, const long dims1[D], const long strs1[D], void
  */
 void md_circ_ext(unsigned int D, const long dims1[D],  void* dst, const long dims2[D], const void* src, size_t size)
 {
-	long strs1[D];
-	long strs2[D];
-	md_calc_strides(D, strs1, dims1, size);
-	md_calc_strides(D, strs2, dims2, size);
-	md_circ_ext2(D, dims1, strs1, dst, dims2, strs2, src, size);
+	md_circ_ext2(D, dims1, MD_STRIDES(D, dims1, size), dst,
+			dims2, MD_STRIDES(D, dims2, size), src, size);
 }
 
 
 
-
-
 /**
  * Periodically extend array (with strides)
  *
@@ -1473,7 +1588,7 @@ void md_periodic2(unsigned int D, const long dims1[D], const long strs1[D], void
 
 		assert(0 == dims1[i] % dims2[i]);
 
-		// blocks	
+		// blocks
 		dims1B[2 * i + 0] = dims2[i];
 		strs1B[2 * i + 0] = strs1[i];
 
@@ -1497,11 +1612,8 @@ void md_periodic2(unsigned int D, const long dims1[D], const long strs1[D], void
  */
 void md_periodic(unsigned int D, const long dims1[D], void* dst, const long dims2[D], const void* src, size_t size)
 {
-	long strs1[D];
-	long strs2[D];
-	md_calc_strides(D, strs1, dims1, size);
-	md_calc_strides(D, strs2, dims2, size);
-	md_periodic2(D, dims1, strs1, dst, dims2, strs2, src, size);
+	md_periodic2(D, dims1, MD_STRIDES(D, dims1, size), dst,
+			dims2, MD_STRIDES(D, dims2, size), src, size);
 }
 
 
@@ -1526,7 +1638,9 @@ void* md_alloc(unsigned int D, const long dimensions[D], size_t size)
 void* md_calloc(unsigned int D, const long dimensions[D], size_t size)
 {
 	void* ptr = md_alloc(D, dimensions, size);
+
 	md_clear(D, dimensions, ptr, size);
+
 	return ptr;
 }
 
@@ -1545,7 +1659,6 @@ void* md_alloc_gpu(unsigned int D, const long dimensions[D], size_t size)
 
 
 
-
 /**
  * Allocate GPU memory and copy from CPU pointer
  *
@@ -1557,7 +1670,9 @@ void* md_gpu_move(unsigned int D, const long dims[D], const void* ptr, size_t si
 		return NULL;
 
 	void* gpu_ptr = md_alloc_gpu(D, dims, size);
+
 	md_copy(D, dims, gpu_ptr, ptr, size);
+
 	return gpu_ptr;
 }
 #endif
@@ -1585,14 +1700,12 @@ void* md_alloc_sameplace(unsigned int D, const long dimensions[D], size_t size,
  * Free CPU/GPU memory
  *
  */
-void md_free(void* ptr)
+void md_free(const void* ptr)
 {
 #ifdef USE_CUDA
 	if (cuda_ondevice(ptr))
-		cuda_free(ptr);
+		cuda_free((void*)ptr);
 	else
 #endif
-	free(ptr);
+	xfree(ptr);
 }
-
-
