@@ -1,9 +1,10 @@
 /* Copyright 2015. The Regents of the University of California.
+ * Copyright 2016. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors:
- * 2013-2015 Martin Uecker <uecker@eecs.berkeley.edu>
+ * 2013-2016 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  * 2014 Jonathan Tamir <jtamir@eecs.berkeley.edu>
  * 2014 Frank Ong <frankong@berkeley.edu>
  *
@@ -15,13 +16,16 @@
 #include <stdlib.h>
 #include <assert.h>
 #include <string.h>
+#include <stdio.h>
 #include <alloca.h>
 
 #include "num/multind.h"
 #include "num/iovec.h"
 
 #include "misc/misc.h"
+#include "misc/types.h"
 #include "misc/debug.h"
+#include "misc/shrdptr.h"
 
 #include "ops.h"
 
@@ -35,40 +39,56 @@
 struct operator_s {
 
 	unsigned int N;
+	unsigned int io_flags;
 	const struct iovec_s** domain;
 
-	void* data;
-	int refcount;
+	operator_data_t* data;
+	void (*apply)(const operator_data_t* data, unsigned int N, void* args[N]);
+	void (*del)(const operator_data_t* data);
 
-	void (*apply)(const void* data, unsigned int N, void* args[N]);
-	void (*del)(const void* data);
+	struct shared_obj_s sptr;
 };
 
 
+static void operator_del(const struct shared_obj_s* sptr)
+{
+	const struct operator_s* x = CONTAINER_OF(sptr, const struct operator_s, sptr);
+
+	if (NULL != x->del)
+		x->del(x->data);
+
+	for (unsigned int i = 0; i < x->N; i++)
+		iovec_free(x->domain[i]);
+
+	xfree(x->domain);
+	xfree(x);
+}
+
+
 
 /**
  * Create an operator (with strides)
  */
-const struct operator_s* operator_generic_create2(unsigned int N, const unsigned int D[N],
-			const long* dims[N], const long* strs[N],
-			void* data, operator_fun_t apply, operator_del_t del)
+const struct operator_s* operator_generic_create2(unsigned int N, unsigned int io_flags,
+			const unsigned int D[N], const long* dims[N], const long* strs[N],
+			operator_data_t* data, operator_fun_t apply, operator_del_t del)
 {
-	struct operator_s* op = xmalloc(sizeof(struct operator_s));
-
-	const struct iovec_s** dom = xmalloc(N * sizeof(struct iovec_s*));
+	PTR_ALLOC(struct operator_s, op);
+	PTR_ALLOC(const struct iovec_s*[N], dom);
 
 	for (unsigned int i = 0; i < N; i++)
-		dom[i] = iovec_create2(D[i], dims[i], strs[i], CFL_SIZE);
+		(*dom)[i] = iovec_create2(D[i], dims[i], strs[i], CFL_SIZE);
 
 	op->N = N;
-	op->domain = dom;
+	op->io_flags = io_flags;
+	op->domain = *PTR_PASS(dom);
 	op->data = data;
 	op->apply = apply;
-
-	op->refcount = 1;
 	op->del = del;
 
-	return op;
+	shared_obj_init(&op->sptr, operator_del);
+
+	return PTR_PASS(op);
 }
 
 
@@ -76,19 +96,16 @@ const struct operator_s* operator_generic_create2(unsigned int N, const unsigned
 /**
  * Create an operator (without strides)
  */
-const struct operator_s* operator_generic_create(unsigned int N, const unsigned int D[N],
-			const long* dims[N], void* data, operator_fun_t apply, operator_del_t del)
+const struct operator_s* operator_generic_create(unsigned int N, unsigned int io_flags,
+			const unsigned int D[N], const long* dims[N],
+			operator_data_t* data, operator_fun_t apply, operator_del_t del)
 {
 	const long* strs[N];
 
-	for (unsigned int i = 0; i < N; i++) {
+	for (unsigned int i = 0; i < N; i++)
+		strs[i] = MD_STRIDES(D[i], dims[i], CFL_SIZE);
 
-		long* ptr = alloca(D[i] * sizeof(long));
-		md_calc_strides(D[i], ptr, dims[i], CFL_SIZE);
-		strs[i] = ptr;
-	}
-
-	return operator_generic_create2(N, D, dims, strs, data, apply, del);
+	return operator_generic_create2(N, io_flags, D, dims, strs, data, apply, del);
 }
 
 
@@ -98,10 +115,11 @@ const struct operator_s* operator_generic_create(unsigned int N, const unsigned
  */
 const struct operator_s* operator_create2(unsigned int ON, const long out_dims[ON], const long out_strs[ON],
 			unsigned int IN, const long in_dims[IN], const long in_strs[IN],
-			void* data, operator_fun_t apply, operator_del_t del)
+			operator_data_t* data, operator_fun_t apply, operator_del_t del)
 {
-	return operator_generic_create2(2, (unsigned int[2]){ ON, IN }, (const long* [2]){ out_dims, in_dims },
-			(const long* [2]){ out_strs, in_strs }, data, apply, del);
+	return operator_generic_create2(2, MD_BIT(0), (unsigned int[2]){ ON, IN },
+				(const long* [2]){ out_dims, in_dims }, (const long* [2]){ out_strs, in_strs },
+				data, apply, del);
 }
 
 /**
@@ -117,14 +135,11 @@ const struct operator_s* operator_create2(unsigned int ON, const long out_dims[O
  */
 const struct operator_s* operator_create(unsigned int ON, const long out_dims[ON], 
 		unsigned int IN, const long in_dims[IN],
-		void* data, operator_fun_t apply, operator_del_t del)
+		operator_data_t* data, operator_fun_t apply, operator_del_t del)
 {
-	long out_strs[ON];
-	long in_strs[IN];
-	md_calc_strides(ON, out_strs, out_dims, CFL_SIZE);
-	md_calc_strides(IN, in_strs, in_dims, CFL_SIZE);
-
-	return operator_create2(ON, out_dims, out_strs, IN, in_dims, in_strs, data, apply, del);
+	return operator_create2(ON, out_dims, MD_STRIDES(ON, out_dims, CFL_SIZE),
+				IN, in_dims, MD_STRIDES(IN, in_dims, CFL_SIZE),
+				data, apply, del);
 }
 
 
@@ -136,7 +151,7 @@ const struct operator_s* operator_create(unsigned int ON, const long out_dims[ON
 const struct operator_s* operator_ref(const struct operator_s* x)
 {
 	if (NULL != x)
-		((struct operator_s*)x)->refcount++;
+		shared_obj_ref(&x->sptr);
 
 	return x;
 }
@@ -148,7 +163,7 @@ const struct operator_s* operator_ref(const struct operator_s* x)
  *
  * @param x operator
  */
-void* operator_get_data(const struct operator_s* x)
+operator_data_t* operator_get_data(const struct operator_s* x)
 {
 	return x->data;
 }
@@ -166,17 +181,7 @@ void operator_free(const struct operator_s* x)
 	if (NULL == x) 
 		return;
 
-	if (1 > --(((struct operator_s*)x)->refcount)) {
-
-		if (NULL != x->del)
-			x->del(x->data);
-
-		for (unsigned int i = 0; i < x->N; i++)
-			iovec_free(x->domain[i]);
-
-		free(x->domain);
-		free((void*)x);
-	}
+	shared_obj_destroy(&x->sptr);
 }
 
 
@@ -272,28 +277,32 @@ void operator_p_free(const struct operator_p_s* x)
 
 struct op_p_data_s {
 
-	void* data;
+	INTERFACE(operator_data_t);
+
+	operator_data_t* data;
 	operator_p_fun_t apply;
 	operator_del_t del;
 };
 
-static void op_p_apply(const void* _data, unsigned int N, void* args[N])
+DEF_TYPEID(op_p_data_s);
+
+static void op_p_apply(const operator_data_t* _data, unsigned int N, void* args[N])
 {
-	const struct op_p_data_s* data = _data;
+	const struct op_p_data_s* data = CAST_DOWN(op_p_data_s, _data);
 	assert(3 == N);
 	data->apply(data->data, *((float*)args[0]), args[1], args[2]);
 }
 
-static void op_p_del(const void* _data)
+static void op_p_del(const operator_data_t* _data)
 {
-	const struct op_p_data_s* data = _data;
+	const struct op_p_data_s* data = CAST_DOWN(op_p_data_s, _data);
 	data->del(data->data);
 	free((void*)data);
 }
 
-void* operator_p_get_data(const struct operator_p_s* x)
+operator_data_t* operator_p_get_data(const struct operator_p_s* _data)
 {
-	struct op_p_data_s* data = operator_get_data(&x->op);
+	const struct op_p_data_s* data = CAST_DOWN(op_p_data_s, operator_get_data(&_data->op));
 	return data->data;
 }
 
@@ -302,33 +311,35 @@ void* operator_p_get_data(const struct operator_p_s* x)
  */
 const struct operator_p_s* operator_p_create2(unsigned int ON, const long out_dims[ON], const long out_strs[ON], 
 		unsigned int IN, const long in_dims[IN], const long in_strs[IN],
-		void* data, operator_p_fun_t apply, operator_del_t del)
+		operator_data_t* data, operator_p_fun_t apply, operator_del_t del)
 {
-	struct operator_p_s* o = xmalloc(sizeof(struct operator_p_s));
-	struct op_p_data_s* op = xmalloc(sizeof(struct op_p_data_s));
+	PTR_ALLOC(struct operator_p_s, o);
+	PTR_ALLOC(struct op_p_data_s, op);
+	SET_TYPEID(op_p_data_s, op);
 
 	op->data = data;
 	op->apply = apply;
 	op->del = del;
 
-	const struct iovec_s** dom = xmalloc(3 * sizeof(struct iovec_s*));
+	PTR_ALLOC(const struct iovec_s*[3], dom);
 
-	dom[0] = iovec_create2(1, MD_DIMS(1), MD_DIMS(0), FL_SIZE);
-	dom[1] = iovec_create2(ON, out_dims, out_strs, CFL_SIZE);
-	dom[2] = iovec_create2(IN, in_dims, in_strs, CFL_SIZE);
+	(*dom)[0] = iovec_create2(1, MD_DIMS(1), MD_DIMS(0), FL_SIZE);
+	(*dom)[1] = iovec_create2(ON, out_dims, out_strs, CFL_SIZE);
+	(*dom)[2] = iovec_create2(IN, in_dims, in_strs, CFL_SIZE);
 
 	o->op.N = 3;
-	o->op.domain = dom;
-	o->op.data = op;
+	o->op.io_flags = MD_BIT(1);
+	o->op.domain = *PTR_PASS(dom);
+	o->op.data = CAST_UP(PTR_PASS(op));
 	o->op.apply = op_p_apply;
-
-	o->op.refcount = 1;
 	o->op.del = op_p_del;
 
+	shared_obj_init(&o->op.sptr, operator_del);
+
 	if (NULL == del)
 		debug_printf(DP_WARN, "Warning: no delete function specified for operator_p_create! Possible memory leak.\n");
 
-	return o;
+	return PTR_PASS(o);
 }
 
 
@@ -345,35 +356,35 @@ const struct operator_p_s* operator_p_create2(unsigned int ON, const long out_di
  */
 const struct operator_p_s* operator_p_create(unsigned int ON, const long out_dims[ON], 
 		unsigned int IN, const long in_dims[IN], 
-		void* data, operator_p_fun_t apply, operator_del_t del)
+		operator_data_t* data, operator_p_fun_t apply, operator_del_t del)
 {
-	long out_strs[ON];
-	long in_strs[IN];
-	md_calc_strides(ON, out_strs, out_dims, CFL_SIZE);
-	md_calc_strides(IN, in_strs, in_dims, CFL_SIZE);
-
-	return operator_p_create2(ON, out_dims, out_strs, IN, in_dims, in_strs, data, apply, del);
+	return operator_p_create2(ON, out_dims, MD_STRIDES(ON, out_dims, CFL_SIZE),
+				IN, in_dims, MD_STRIDES(IN, in_dims, CFL_SIZE),
+				data, apply, del);
 }
 
 
 struct identity_s {
 
+	INTERFACE(operator_data_t);
+
 	const struct iovec_s* domain;
 	const struct iovec_s* codomain;
 };
 
-static void identity_apply(const void* _data, unsigned int N, void* args[N])
+DEF_TYPEID(identity_s);
+
+static void identity_apply(const operator_data_t* _data, unsigned int N, void* args[N])
 {
-        const struct identity_s* d = _data;
+        const struct identity_s* d = CAST_DOWN(identity_s, _data);
 	assert(2 == N);
         md_copy2(d->domain->N, d->domain->dims, d->codomain->strs, args[0], d->domain->strs, args[1], d->domain->size);
 }
 
 
-static void identity_free(const void* _data)
+static void identity_free(const operator_data_t* _data)
 {
-        const struct identity_s* d = _data;
-        iovec_free(d->domain);
+        const struct identity_s* d = CAST_DOWN(identity_s, _data);
         iovec_free(d->codomain);
 	free((void*)d);
 }
@@ -382,11 +393,13 @@ static void identity_free(const void* _data)
 const struct operator_s* operator_identity_create2(unsigned int N, const long dims[N],
 					const long ostrs[N], const long istrs[N])
 {
-	struct identity_s* data = xmalloc(sizeof(struct identity_s));
+	PTR_ALLOC(struct identity_s, data);
+	SET_TYPEID(identity_s, data);
+
         data->domain = iovec_create2(N, dims, istrs, CFL_SIZE);
         data->codomain = iovec_create2(N, dims, ostrs, CFL_SIZE);
 
-        return operator_create2(N, dims, ostrs, N, dims, istrs, data, identity_apply, identity_free);
+        return operator_create2(N, dims, ostrs, N, dims, istrs, CAST_UP(PTR_PASS(data)), identity_apply, identity_free);
 }
 
 /**
@@ -404,14 +417,18 @@ const struct operator_s* operator_identity_create(unsigned int N, const long dim
 
 struct operator_chain_s {
 
+	INTERFACE(operator_data_t);
+
 	const struct operator_s* a;
 	const struct operator_s* b;
 };
 
+DEF_TYPEID(operator_chain_s);
 
-static void chain_apply(const void* _data, unsigned int N, void* args[N])
+
+static void chain_apply(const operator_data_t* _data, unsigned int N, void* args[N])
 {
-	const struct operator_chain_s* data = _data;
+	const struct operator_chain_s* data = CAST_DOWN(operator_chain_s, _data);
 
 	assert(2 == N);
 	assert(2 == data->a->N);
@@ -429,9 +446,9 @@ static void chain_apply(const void* _data, unsigned int N, void* args[N])
 /*
  * Free data associated with chained operator
  */
-static void chain_free(const void* _data)
+static void chain_free(const operator_data_t* _data)
 {
-	const struct operator_chain_s* data = _data;
+	const struct operator_chain_s* data = CAST_DOWN(operator_chain_s, _data);
 
 	operator_free(data->a);
 	operator_free(data->b);
@@ -447,11 +464,13 @@ static void chain_free(const void* _data)
  */
 const struct operator_s* operator_chain(const struct operator_s* a, const struct operator_s* b)
 {
-	struct operator_chain_s* c = xmalloc(sizeof(struct operator_chain_s));
+	PTR_ALLOC(struct operator_chain_s, c);
+	SET_TYPEID(operator_chain_s, c);
 	
 	// check compatibility
 
 	assert((2 == a->N) && (2 == b->N));
+	assert((MD_BIT(0) == a->io_flags) && (MD_BIT(0) == b->io_flags));
 	assert(a->domain[0]->N == b->domain[1]->N);
 	assert(md_calc_size(a->domain[0]->N, a->domain[0]->dims) == md_calc_size(b->domain[1]->N, b->domain[1]->dims));
 
@@ -465,7 +484,7 @@ const struct operator_s* operator_chain(const struct operator_s* a, const struct
 
 	const struct iovec_s* dom = a->domain[1];
 	const struct iovec_s* cod = b->domain[0];
-	return operator_create2(cod->N, cod->dims, cod->strs, dom->N, dom->dims, dom->strs, c, chain_apply, chain_free);
+	return operator_create2(cod->N, cod->dims, cod->strs, dom->N, dom->dims, dom->strs, CAST_UP(PTR_PASS(c)), chain_apply, chain_free);
 }
 
 
@@ -486,6 +505,8 @@ const struct operator_s* operator_chainN(unsigned int N, const struct operator_s
 
 struct operator_stack_s {
 
+	INTERFACE(operator_data_t);
+
 	const struct operator_s* a;
 	const struct operator_s* b;
 
@@ -493,19 +514,21 @@ struct operator_stack_s {
 	long src_offset;
 };
 
+DEF_TYPEID(operator_stack_s);
 
-static void stack_apply(const void* _data, unsigned int N, void* args[N])
+
+static void stack_apply(const operator_data_t* _data, unsigned int N, void* args[N])
 {
-	const struct operator_stack_s* data = _data;
+	const struct operator_stack_s* data = CAST_DOWN(operator_stack_s, _data);
 	assert(2 == N);
 
 	operator_apply_unchecked(data->a, args[0], args[1]);
 	operator_apply_unchecked(data->b, args[0] + data->dst_offset, args[1] + data->src_offset);
 }
 
-static void stack_free(const void* _data)
+static void stack_free(const operator_data_t* _data)
 {
-	const struct operator_stack_s* data = _data;
+	const struct operator_stack_s* data = CAST_DOWN(operator_stack_s, _data);
 
 	operator_free(data->a);
 	operator_free(data->b);
@@ -535,7 +558,7 @@ static void stack_dims(unsigned int N, long dims[N], long strs[N], unsigned int
 	md_copy_dims(N, dims, a->dims);
 	md_copy_strides(N, strs, a->strs);
 
-	UNUSED( b );
+	UNUSED(b);
 
 	strs[D] = md_calc_size(N, a->dims) * CFL_SIZE;	// FIXME
 	dims[D] = 2;
@@ -546,7 +569,8 @@ static void stack_dims(unsigned int N, long dims[N], long strs[N], unsigned int
  */
 const struct operator_s* operator_stack(unsigned int D, unsigned int E, const struct operator_s* a, const struct operator_s* b)
 {
-	struct operator_stack_s* c = xmalloc(sizeof(struct operator_stack_s));
+	PTR_ALLOC(struct operator_stack_s, c);
+	SET_TYPEID(operator_stack_s, c);
 
 	assert(stack_compatible(D, a->domain[0], b->domain[0]));
 	assert(stack_compatible(E, a->domain[1], b->domain[1]));
@@ -569,14 +593,16 @@ const struct operator_s* operator_stack(unsigned int D, unsigned int E, const st
 	c->dst_offset = cod_strs[D];
 	c->src_offset = dom_strs[D];
 
-	return operator_create2(cod_N, cod_dims, cod_strs, dom_N, dom_dims, dom_strs, c, stack_apply, stack_free);
+	return operator_create2(cod_N, cod_dims, cod_strs, dom_N, dom_dims, dom_strs, CAST_UP(PTR_PASS(c)), stack_apply, stack_free);
 }
 
 
 
 void operator_generic_apply_unchecked(const struct operator_s* op, unsigned int N, void* args[N])
 {
-	op->apply((void*)op->data, N, args);
+	debug_trace("ENTER %p\n", op->apply);
+	op->apply(op->data, N, args);
+	debug_trace("LEAVE %p\n", op->apply);
 }
 
 
@@ -585,7 +611,7 @@ void operator_apply_unchecked(const struct operator_s* op, complex float* dst, c
 	operator_generic_apply_unchecked(op, 2, (void*[2]){ (void*)dst, (void*)src });
 }
 
-void operator_apply2(const struct operator_s* op, unsigned int IN, const long idims[IN], const long istrs[IN], complex float* dst, const long ON, const long odims[ON], const long ostrs[ON], const complex float* src)
+void operator_apply2(const struct operator_s* op, unsigned int ON, const long odims[ON], const long ostrs[ON], complex float* dst, const long IN, const long idims[IN], const long istrs[ON], const complex float* src)
 {
 	assert(2 == op->N);
 	assert(iovec_check(op->domain[1], IN, idims, istrs));
@@ -594,17 +620,14 @@ void operator_apply2(const struct operator_s* op, unsigned int IN, const long id
 	operator_apply_unchecked(op, dst, src);
 }
 
-void operator_apply(const struct operator_s* op, unsigned int IN, const long idims[IN], complex float* dst, const long ON, const long odims[ON], const complex float* src)
+void operator_apply(const struct operator_s* op, unsigned int ON, const long odims[ON], complex float* dst, const long IN, const long idims[IN], const complex float* src)
 {
-	long ostrs[ON];
-	long istrs[IN];
-	md_calc_strides(ON, ostrs, odims, CFL_SIZE);
-	md_calc_strides(IN, istrs, idims, CFL_SIZE);
-	operator_apply2(op, ON, odims, ostrs, dst, IN, idims, istrs, src);
+	operator_apply2(op, ON, odims, MD_STRIDES(ON, odims, CFL_SIZE), dst,
+			    IN, idims, MD_STRIDES(IN, idims, CFL_SIZE), src);
 }
 
 
-void operator_p_apply2(const struct operator_p_s* op, float mu, unsigned int IN, const long idims[IN], const long istrs[IN], complex float* dst, const long ON, const long odims[ON], const long ostrs[ON], const complex float* src)
+void operator_p_apply2(const struct operator_p_s* op, float mu, unsigned int ON, const long odims[ON], const long ostrs[ON], complex float* dst, const long IN, const long idims[IN], const long istrs[IN], const complex float* src)
 {
 	assert(3 == op->op.N);
 	assert(iovec_check(op->op.domain[2], IN, idims, istrs));
@@ -614,13 +637,11 @@ void operator_p_apply2(const struct operator_p_s* op, float mu, unsigned int IN,
 }
 
 
-void operator_p_apply(const struct operator_p_s* op, float mu, unsigned int IN, const long idims[IN], complex float* dst, const long ON, const long odims[ON], const complex float* src)
+void operator_p_apply(const struct operator_p_s* op, float mu, unsigned int ON, const long odims[ON], complex float* dst, const long IN, const long idims[IN], const complex float* src)
 {
-	long ostrs[ON];
-	long istrs[IN];
-	md_calc_strides(ON, ostrs, odims, CFL_SIZE);
-	md_calc_strides(IN, istrs, idims, CFL_SIZE);
-	operator_p_apply2(op, mu, ON, odims, ostrs, dst, IN, idims, istrs, src);
+	operator_p_apply2(op, mu,
+			ON, odims, MD_STRIDES(ON, odims, CFL_SIZE), dst,
+			IN, idims, MD_STRIDES(IN, idims, CFL_SIZE), src);
 }
 
 
@@ -647,18 +668,260 @@ void operator_p_iter(void* o, float lambda, float* _dst, const float* _src)
 }
 
 
-#ifdef USE_CUDA
-static void gpuwrp_fun(const void* _data, unsigned int N, void* args[N])
+
+struct op_loop_s {
+
+	INTERFACE(operator_data_t);
+
+	unsigned int N;
+	unsigned int D;
+	const long** strs;
+	const long** dims;
+	const long* dims0;
+	const struct operator_s* op;
+};
+
+DEF_TYPEID(op_loop_s);
+
+static void op_loop_del(const operator_data_t* _data)
 {
-	const struct operator_s* op = _data;
-	void* gpu_ptr[N];
+	const struct op_loop_s* data = CAST_DOWN(op_loop_s, _data);
+	operator_free(data->op);
+
+	for (unsigned int i = 0; i < data->N; i++) {
+
+		free((void*)data->dims[i]);
+		free((void*)data->strs[i]);
+	}
+
+	free((void*)data->strs);
+	free((void*)data->dims);
+	free((void*)data->dims0);
+	free((void*)data);
+}
+
+static void op_loop_nary(void* _data, void* ptr[])
+{
+	const struct op_loop_s* data = _data;
+	operator_generic_apply_unchecked(data->op, data->N, ptr);
+}
+
+static void op_loop_fun(const operator_data_t* _data, unsigned int N, void* args[N])
+{
+	const struct op_loop_s* data = CAST_DOWN(op_loop_s, _data);
+	assert(N == data->N);
+	md_nary(N, data->D, data->dims0, data->strs, args, (void*)data, op_loop_nary);
+}
+
+static void merge_dims(unsigned int D, long odims[D], const long idims1[D], const long idims2[D])
+{
+	md_copy_dims(D, odims, idims1);
+
+	for (unsigned int i = 0; i < D; i++) {
+
+		assert((1 == odims[i]) | (1 == idims2[i]));
+
+		if (1 == odims[i])
+			odims[i] = idims2[i];
+	}
+}
+
+const struct operator_s* (operator_loop2)(unsigned int N, const unsigned int D,
+				const long dims[D], const long (*strs)[D],
+				const struct operator_s* op)
+{
+	assert(N == operator_nr_args(op));
+
+	unsigned int D2[N];
+	PTR_ALLOC(long[D], dims0);
+	md_copy_dims(D, *dims0, dims);
+
+	PTR_ALLOC(const long*[N], dims2);
+	PTR_ALLOC(const long*[N], strs2);
+
+
+	// TODO: we should have a flag and ignore args with flag
+
+	for (unsigned int i = 0; i < N; i++) {
+
+		const struct iovec_s* io = operator_arg_domain(op, i);
+
+		assert(D == io->N);
+
+		for (unsigned int j = 0; j < D; j++) {
+
+			assert((0 == io->strs[j]) || (io->strs[j] == strs[i][j]));
+			assert((1 == io->dims[j]) == (0 == io->strs[j]));
+		}
+
+		D2[i] = D;
+
+		PTR_ALLOC(long[D], tdims);
+		merge_dims(D, *tdims, dims, io->dims);
+
+		PTR_ALLOC(long[D], tstrs);
+		md_copy_strides(D, *tstrs, strs[i]);
+
+		(*dims2)[i] = *PTR_PASS(tdims);
+		(*strs2)[i] = *PTR_PASS(tstrs);
+	}
+
+	PTR_ALLOC(struct op_loop_s, data);
+	SET_TYPEID(op_loop_s, data);
+	data->N = N;
+	data->D = D;
+	data->op = op;
+
+	data->dims0 = *PTR_PASS(dims0);
+	data->dims = */*PTR_PASS*/(dims2);
+	data->strs = */*PTR_PASS*/(strs2);
+
+	const struct operator_s* rop = operator_generic_create2(N, op->io_flags, D2, *dims2, *strs2, CAST_UP(PTR_PASS(data)), op_loop_fun, op_loop_del);
+
+	PTR_PASS(dims2);
+	PTR_PASS(strs2);
+
+	return rop;
+}
+
+const struct operator_s* operator_loop(unsigned int D, const long dims[D], const struct operator_s* op)
+{
+	unsigned int N = operator_nr_args(op);
+	long strs[N][D];
+
+	for (unsigned int i = 0; i < N; i++) {
+
+		long tdims[D];
+		merge_dims(D, tdims, dims, operator_arg_domain(op, i)->dims);
+		md_calc_strides(D, strs[i], tdims, operator_arg_domain(op, i)->size);
+	}
+
+	return operator_loop2(N, D, dims, strs, op);
+}
+
+struct copy_data_s {
+
+	INTERFACE(operator_data_t);
+
+	const struct operator_s* op;
+
+	unsigned int N;
+	const long** strs;
+};
+
+DEF_TYPEID(copy_data_s);
+
+static void copy_fun(const operator_data_t* _data, unsigned int N, void* args[N])
+{
+	const struct copy_data_s* data = CAST_DOWN(copy_data_s, _data);
+	const struct operator_s* op = data->op;
+	void* ptr[N];
 
 	assert(N == operator_nr_args(op));
 
 	for (unsigned int i = 0; i < N; i++) {
 
 		const struct iovec_s* io = operator_arg_domain(op, i);
-		gpu_ptr[i] = md_gpu_move(io->N, io->dims, args[i], io->size);
+
+		ptr[i] = md_alloc(io->N, io->dims, io->size);
+
+		if (!MD_IS_SET(op->io_flags, i))
+			md_copy2(io->N, io->dims, io->strs, ptr[i], data->strs[i], args[i], io->size);
+	}
+
+	operator_generic_apply_unchecked(op, N, ptr);
+
+	for (unsigned int i = 0; i < N; i++) {
+
+		const struct iovec_s* io = operator_arg_domain(op, i);
+
+		if (MD_IS_SET(op->io_flags, i))
+			md_copy2(io->N, io->dims, data->strs[i], args[i], io->strs, ptr[i], io->size);
+
+		md_free(ptr[i]);
+	}
+}
+
+static void copy_del(const operator_data_t* _data)
+{
+	const struct copy_data_s* data = CAST_DOWN(copy_data_s, _data);
+
+	operator_free(data->op);
+
+	for (unsigned int i = 0; i < data->N; i++)
+		xfree(data->strs[i]);
+
+	xfree(data->strs);
+	xfree(data);
+}
+
+const struct operator_s* operator_copy_wrapper(unsigned int N, const long* strs[N], const struct operator_s* op)
+{
+	assert(N == operator_nr_args(op));
+
+	// op = operator_ref(op);
+	PTR_ALLOC(struct copy_data_s, data);
+	SET_TYPEID(copy_data_s, data);
+	data->op = op;
+
+	unsigned int D[N];
+	const long* dims[N];
+	const long* (*strs2)[N] = TYPE_ALLOC(const long*[N]);
+
+	for (unsigned int i = 0; i < N; i++) {
+
+		const struct iovec_s* io = operator_arg_domain(op, i);
+
+		D[i] = io->N;
+		dims[i] = io->dims;
+
+		long (*strsx)[io->N] = TYPE_ALLOC(long[io->N]);
+		md_copy_strides(io->N, *strsx, strs[i]);
+		(*strs2)[i] = *strsx;
+
+		long tstrs[io->N];
+		md_calc_strides(io->N, tstrs, io->dims, CFL_SIZE);
+
+		for (unsigned int i = 0; i < io->N; i++)
+			assert(io->strs[i] == tstrs[i]);
+	}
+
+	data->N = N;
+	data->strs = *strs2;
+
+	return operator_generic_create2(N, op->io_flags, D, dims, *strs2, CAST_UP(PTR_PASS(data)), copy_fun, copy_del);
+}
+
+
+
+
+struct gpu_data_s {
+
+	INTERFACE(operator_data_t);
+
+	const struct operator_s* op;
+};
+
+DEF_TYPEID(gpu_data_s);
+
+static void gpuwrp_fun(const operator_data_t* _data, unsigned int N, void* args[N])
+{
+#ifdef USE_CUDA
+	const struct operator_s* op = CAST_DOWN(gpu_data_s, _data)->op;
+	void* gpu_ptr[N];
+
+	assert(N == operator_nr_args(op));
+
+	debug_printf(DP_DEBUG1, "GPU start.\n");
+
+	for (unsigned int i = 0; i < N; i++) {
+
+		const struct iovec_s* io = operator_arg_domain(op, i);
+
+		if (MD_IS_SET(op->io_flags, i))
+			gpu_ptr[i] = md_alloc_gpu(io->N, io->dims, io->size);
+		else
+			gpu_ptr[i] = md_gpu_move(io->N, io->dims, args[i], io->size);
 	}
 
 	operator_generic_apply_unchecked(op, N, gpu_ptr);
@@ -666,15 +929,28 @@ static void gpuwrp_fun(const void* _data, unsigned int N, void* args[N])
 	for (unsigned int i = 0; i < N; i++) {
 
 		const struct iovec_s* io = operator_arg_domain(op, i);
-		md_copy(io->N, io->dims, args[i], gpu_ptr[i], io->size);
+
+		if (MD_IS_SET(op->io_flags, i))
+			md_copy(io->N, io->dims, args[i], gpu_ptr[i], io->size);
+
 		md_free(gpu_ptr[i]);
 	}
+
+	debug_printf(DP_DEBUG1, "GPU end.\n");
+
+#else
+	UNUSED(_data); UNUSED(N); UNUSED(args);
+	assert(0);
+#endif
 }
 
-static void gpuwrp_del(const void* _data)
+static void gpuwrp_del(const operator_data_t* _data)
 {
-	const struct operator_s* op = _data;
-	operator_free(op);
+	const struct gpu_data_s* data = CAST_DOWN(gpu_data_s, _data);
+
+	operator_free(data->op);
+
+	free((void*)data);
 }
 
 const struct operator_s* operator_gpu_wrapper(const struct operator_s* op)
@@ -695,8 +971,10 @@ const struct operator_s* operator_gpu_wrapper(const struct operator_s* op)
 	}
 
 	// op = operator_ref(op);
+	PTR_ALLOC(struct gpu_data_s, data);
+	SET_TYPEID(gpu_data_s, data);
+	data->op = op;
 
-	return operator_generic_create2(N, D, dims, strs, (void*)op, gpuwrp_fun, gpuwrp_del);
+	return operator_generic_create2(N, op->io_flags, D, dims, strs, CAST_UP(PTR_PASS(data)), gpuwrp_fun, gpuwrp_del);
 }
-#endif
 
