@@ -14,14 +14,15 @@
 
 #include "num/multind.h"
 #include "num/flpmath.h"
+#include "num/init.h"
 
 #include "misc/mmio.h"
 #include "misc/io.h"
 #include "misc/misc.h"
 
 
-static const char* usage_str = "dims dim1 ... dimn name";
-static const char* help_str = "Create an array filled with ones with {dims} dimensions of size {dim1} to {dimn}.\n";
+static const char usage_str[] = "dims dim1 ... dimn name";
+static const char help_str[] = "Create an array filled with ones with {dims} dimensions of size {dim1} to {dimn}.\n";
 
 
 
@@ -29,6 +30,8 @@ int main_ones(int argc, char* argv[])
 {
 	mini_cmdline(argc, argv, -3, usage_str, help_str);
 
+	num_init();
+
 	int N = atoi(argv[1]);
 
 	assert(N >= 0);
