@@ -14,6 +14,7 @@
 
 #include "num/multind.h"
 #include "num/flpmath.h"
+#include "num/init.h"
 
 #include "misc/mmio.h"
 #include "misc/misc.h"
@@ -21,14 +22,16 @@
 
 #define DIMS 16
 
-static const char*  usage_str = "dimension repetitions <input> <output>";
-static const char* help_str = "Repeat input array multiple times along a certain dimension.\n";
+static const char usage_str[] = "dimension repetitions <input> <output>";
+static const char help_str[] = "Repeat input array multiple times along a certain dimension.\n";
 
 
 int main_repmat(int argc, char* argv[])
 {
 	mini_cmdline(argc, argv, 4, usage_str, help_str);
 
+	num_init();
+
 	long in_dims[DIMS];
 	long out_dims[DIMS];
 	
