@@ -23,6 +23,7 @@
 #include "num/flpmath.h"
 #include "num/iovec.h"
 #include "num/ops.h"
+#include "num/init.h"
 
 #include "linops/linop.h"
 #include "linops/someops.h"
@@ -42,14 +43,16 @@
 #endif
 
 
-static const char* usage_str = "<lambda> <flags> <input> <output>";
-static const char* help_str = "Perform total variation denoising along dims <flags>.\n";
+static const char usage_str[] = "<lambda> <flags> <input> <output>";
+static const char help_str[] = "Perform total variation denoising along dims <flags>.\n";
 
 	
 int main_rof(int argc, char* argv[])
 {
 	mini_cmdline(argc, argv, 4, usage_str, help_str);
 
+	num_init();
+
 	long dims[DIMS];
 
 	float lambda = atof(argv[1]);
@@ -60,7 +63,7 @@ int main_rof(int argc, char* argv[])
 
 	// TV operator
 
-	const struct linop_s* tv_op = grad_init(DIMS, dims, flags);
+	const struct linop_s* tv_op = linop_grad_create(DIMS, dims, flags);
 //	const struct linop_s* tv_op = linop_identity_create(DIMS, dims);
 
 	struct iter_admm_conf conf = iter_admm_defaults;
@@ -71,10 +74,9 @@ int main_rof(int argc, char* argv[])
 	const struct operator_p_s* thresh_prox = prox_thresh_create(DIMS + 1, linop_codomain(tv_op)->dims, 
 								lambda, MD_BIT(DIMS), false);
 
-	iter2_admm(&conf, linop_identity_create(DIMS, dims)->forward,
-		   1, (const struct operator_p_s* []){ thresh_prox }, (const struct linop_s* []){ tv_op }, NULL, 
-		   2 * md_calc_size(DIMS, dims), (float*)out_data, (const float*)in_data,
-		   NULL, NULL, NULL);
+	iter2_admm(CAST_UP(&conf), linop_identity_create(DIMS, dims)->forward,
+		   1, MAKE_ARRAY(thresh_prox), MAKE_ARRAY(tv_op), NULL,
+		   NULL, 2 * md_calc_size(DIMS, dims), (float*)out_data, (const float*)in_data, NULL);
 
 	linop_free(tv_op);
 	operator_p_free(thresh_prox);
