@@ -13,6 +13,7 @@
 #include <stdio.h>
 
 #include "num/multind.h"
+#include "num/init.h"
 
 #include "misc/mmio.h"
 #include "misc/misc.h"
@@ -21,14 +22,16 @@
 #define DIMS 16
 #endif
 
-static const char* usage_str = "scale <input1> <input2> <output>";
-static const char* help_str = "Multiply input1 with scale factor and add input2.\n";
+static const char usage_str[] = "scale <input1> <input2> <output>";
+static const char help_str[] = "Multiply input1 with scale factor and add input2.\n";
 
 
 int main_saxpy(int argc, char* argv[])
 {
 	mini_cmdline(argc, argv, 4, usage_str, help_str);
 
+	num_init();
+
 	complex float scale;
 
 	if (0 != parse_cfl(&scale, argv[1])) {
