@@ -14,6 +14,7 @@
 
 #include "num/multind.h"
 #include "num/flpmath.h"
+#include "num/init.h"
 
 #include "misc/mmio.h"
 #include "misc/misc.h"
@@ -22,14 +23,16 @@
 #define DIMS 16
 #endif
 
-static const char* usage_str = "factor <input> <output>";
-static const char* help_str = 	"Scale array by {factor}. The scale factor can be a complex number.\n";
+static const char usage_str[] = "factor <input> <output>";
+static const char help_str[] = "Scale array by {factor}. The scale factor can be a complex number.\n";
 
 
 int main_scale(int argc, char* argv[])
 {
 	mini_cmdline(argc, argv, 3, usage_str, help_str);
 
+	num_init();
+
 	complex float scale;
 	// = atof(argv[1]);
 	if (0 != parse_cfl(&scale, argv[1])) {
