@@ -1,4 +1,5 @@
 /* Copyright 2013-2014. The Regents of the University of California.
+ * Copyright 2016. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  */
@@ -13,10 +14,10 @@ extern "C" {
 struct operator_s;
 struct vec_ops;
 
-extern struct linop_s* sense_init(const long max_dims[DIMS], unsigned int sens_flags, const complex float* sens, _Bool gpu);
+extern struct linop_s* sense_init(const long max_dims[DIMS], unsigned int sens_flags, const complex float* sens);
 extern struct linop_s* maps_create(const long max_dims[DIMS], 
-			unsigned int sens_flags, const complex float* sens, bool gpu);
-extern struct linop_s* maps2_create(const long coilim_dims[DIMS], const long maps_dims[DIMS], const long img_dims[DIMS], const complex float* maps, bool use_gpu);
+			unsigned int sens_flags, const complex float* sens);
+extern struct linop_s* maps2_create(const long coilim_dims[DIMS], const long maps_dims[DIMS], const long img_dims[DIMS], const complex float* maps);
 
 
 #ifdef __cplusplus
