@@ -1,9 +1,10 @@
 /* Copyright 2013-2014. The Regents of the University of California.
- * All rights reserved. Use of this source code is governed by 
+ * Copyright 2016. Martin Uecker.
+ * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors: 
- * 2012 Martin Uecker <uecker@eecs.berkeley.edu>
+ * 2012, 2016 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  * 2014 Jonathan Tamir <jtamir@eecs.berkeley.edu>
  *
  *
@@ -17,11 +18,11 @@
 #include <stdbool.h>
 #include <assert.h>
 #include <math.h>
-#include <stdio.h>
 
 #include "misc/misc.h"
 #include "misc/mri.h"
 #include "misc/debug.h"
+#include "misc/types.h"
 
 #include "num/multind.h"
 #include "num/flpmath.h"
@@ -33,6 +34,7 @@
 
 #include "iter/iter.h"
 #include "iter/prox.h"
+#include "iter/monitor.h"
 
 #include "sense/model.h"
 
@@ -42,6 +44,8 @@
 
 struct data {
 
+	INTERFACE(operator_data_t);
+
 	const struct linop_s* sense_op;
 	complex float* tmp;
 	float alpha; // l1 or l2 regularization
@@ -60,10 +64,13 @@ struct data {
 	long strs_pat[DIMS];
 };
 
+DEF_TYPEID(data);
 
-static void xupdate_apply(const void* _data, float mu, complex float* dst, const complex float* src)
+
+static void xupdate_apply(const operator_data_t* _data, float mu, complex float* dst, const complex float* src)
 {
-	const struct data* data = _data;
+	const struct data* data = CAST_DOWN(data, _data);
+
 	UNUSED(mu);
 	md_zsmul(DIMS, data->dims_ksp, dst, src, 1. / (data->alpha == 0 ? 2. : 3.));
 }
@@ -90,9 +97,9 @@ static void robust_consistency(float lambda, const long dims[DIMS], complex floa
 
 
 
-static void sparsity_proj_apply(const void* _data, float mu, complex float* dst, const complex float* src)
+static void sparsity_proj_apply(const operator_data_t* _data, float mu, complex float* dst, const complex float* src)
 {
-	const struct data* data = _data;
+	const struct data* data = CAST_DOWN(data, _data);
 
 	const long* dims = data->dims_ksp;
 
@@ -117,10 +124,10 @@ static void sparsity_proj_apply(const void* _data, float mu, complex float* dst,
 }
 
 
-static void data_consistency_proj_apply(const void* _data, float mu, complex float* dst, const complex float* src)
+static void data_consistency_proj_apply(const operator_data_t* _data, float mu, complex float* dst, const complex float* src)
 {
 	UNUSED(mu);
-	const struct data* data = _data;
+	const struct data* data = CAST_DOWN(data, _data);
 
 	if (-1. != data->lambda)
 		robust_consistency(data->lambda, data->dims_ksp, dst, data->pattern, data->kspace);
@@ -129,11 +136,11 @@ static void data_consistency_proj_apply(const void* _data, float mu, complex flo
 }
 
 
-static void sense_proj_apply(const void* _data, float mu, complex float* dst, const complex float* src)
+static void sense_proj_apply(const operator_data_t* _data, float mu, complex float* dst, const complex float* src)
 {
 	UNUSED(mu);
 
-	const struct data* data = _data;
+	const struct data* data = CAST_DOWN(data, _data);
 
 	// assumes normalized sensitivities
 
@@ -142,7 +149,7 @@ static void sense_proj_apply(const void* _data, float mu, complex float* dst, co
 }
 
 
-static void proj_del(const void* _data)
+static void proj_del(const operator_data_t* _data)
 {
 	UNUSED(_data);
 }
@@ -188,6 +195,7 @@ void pocs_recon2(italgo_fun2_t italgo, void* iconf, const struct linop_s* ops[3]
 	md_calc_strides(DIMS, strs_ksp, dims_ksp, CFL_SIZE);
 
 	struct data data;
+	SET_TYPEID(data, &data);
 
 	data.pattern = pattern;
 	data.kspace = kspace;
@@ -200,7 +208,7 @@ void pocs_recon2(italgo_fun2_t italgo, void* iconf, const struct linop_s* ops[3]
 	md_copy_strides(DIMS, data.strs_ksp, strs_ksp);
 	md_copy_strides(DIMS, data.strs_pat, strs_pat);
 
-	data.sense_op = sense_init(dims, FFT_FLAGS|MAPS_FLAG|COIL_FLAG, maps, use_gpu);
+	data.sense_op = sense_init(dims, FFT_FLAGS|MAPS_FLAG|COIL_FLAG, maps);
 
 	data.thresh = thresh_op;
 
@@ -221,8 +229,7 @@ void pocs_recon2(italgo_fun2_t italgo, void* iconf, const struct linop_s* ops[3]
 	data.fftmod_mat = fftmod_mat;
 
 
-
-	const struct operator_p_s* sense_proj = operator_p_create(DIMS, dims_ksp, DIMS, dims_ksp, (void*)&data, sense_proj_apply, proj_del);
+	const struct operator_p_s* sense_proj = operator_p_create(DIMS, dims_ksp, DIMS, dims_ksp, CAST_UP(&data), sense_proj_apply, proj_del);
 
 	const struct operator_p_s* data_consistency_proj = operator_p_create(DIMS, dims_ksp, DIMS, dims_ksp, (void*)&data, data_consistency_proj_apply, proj_del);
 	
@@ -237,8 +244,10 @@ void pocs_recon2(italgo_fun2_t italgo, void* iconf, const struct linop_s* ops[3]
 
 	const struct operator_p_s* xupdate_op = operator_p_create(DIMS, dims_ksp, DIMS, dims_ksp, (void*)&data, xupdate_apply, proj_del);
 
+	long size = 2 * md_calc_size(DIMS, dims_ksp);
+
 	md_clear(DIMS, dims_ksp, result, CFL_SIZE);
-	italgo(iconf, NULL, (alpha == 0.) ? 2 : 3, prox_ops, ops, xupdate_op, md_calc_size(DIMS, dims_ksp) * 2, (float*)result, NULL, NULL, (void*)&data, compute_norm);
+	italgo(iconf, NULL, (alpha == 0.) ? 2 : 3, prox_ops, ops, NULL, xupdate_op, size, (float*)result, NULL, create_monitor(size, NULL, (void*)&data, compute_norm));
 
 	debug_printf(DP_INFO, "Done\n");
 
