@@ -13,6 +13,7 @@
 #include <complex.h>
 
 #include "num/multind.h"
+#include "num/init.h"
 
 #include "misc/mmio.h"
 #include "misc/misc.h"
@@ -28,14 +29,16 @@
 #endif
 
 
-static const char* usage_str = "dimension position <input> <output>";
-static const char* help_str = "Extracts a slice from {position} along {dimension}.\n";
+static const char usage_str[] = "dimension position <input> <output>";
+static const char help_str[] = "Extracts a slice from {position} along {dimension}.\n";
 
 
 int main_slice(int argc, char* argv[])
 {
 	mini_cmdline(argc, argv, 4, usage_str, help_str);
 
+	num_init();
+
 	long in_dims[DIMS];
 	long out_dims[DIMS];
 	
