@@ -14,6 +14,7 @@
 
 #include "num/multind.h"
 #include "num/flpmath.h"
+#include "num/init.h"
 
 #include "misc/mmio.h"
 #include "misc/misc.h"
@@ -22,14 +23,16 @@
 #define DIMS 16
 #endif
 
-static const char* usage_str = "exponent <input> <output>";
-static const char* help_str = "Raise array to the power of {exponent}. The exponent can be a complex number.\n";
+static const char usage_str[] = "exponent <input> <output>";
+static const char help_str[] = "Raise array to the power of {exponent}. The exponent can be a complex number.\n";
 
 
 int main_spow(int argc, char* argv[argc])
 {
 	mini_cmdline(argc, argv, 3, usage_str, help_str);
 
+	num_init();
+
 	complex float expo;
 
 	if (0 != parse_cfl(&expo, argv[1])) {
