@@ -1,15 +1,16 @@
 /* Copyright 2013-2014. The Regents of the University of California.
- * All rights reserved. Use of this source code is governed by 
+ * Copyright 2016. Martin Uecker.
+ * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors: 
  * 20XX-2013 Frank Ong, Martin Uecker, Pat Virtue, and Mark Murphy
  * frankong@berkeley.edu
+ * 2016	     Martin Uecker <martin.uecker@med.uni-goettingen.de>
  */
 
 #define _GNU_SOURCE
 #include <math.h>
-#include <stdio.h>
 #include <string.h>
 #include <assert.h>
 #include <complex.h>
@@ -49,7 +50,7 @@ static void conv_up_3d(data_t *out, const data_t *in, int size1, int skip1, int
 static void create_numLevels(struct wavelet_plan_s* plan);
 static void create_wavelet_sizes(struct wavelet_plan_s* plan);
 
-static void wavelet_del(const void* data);
+static void wavelet_del(const linop_data_t* data);
 
 
 struct wavelet_plan_s* prepare_wavelet_plan_filters(int numdims, const long imSize[numdims], unsigned int flags, const long minSize[numdims], int use_gpu, int filter_length, const float filter[4][filter_length])
@@ -146,65 +147,31 @@ struct wavelet_plan_s* prepare_wavelet_plan(int numdims, const long imSize[numdi
 	return prepare_wavelet_plan_filters(numdims, imSize, flags, minSize, use_gpu, 4, wavelet2_dau2);
 }
 
-static void wavelet_normal(const void* _data, data_t* out, const data_t* _in)
+
+
+
+
+struct wavelet_data_s {
+
+	INTERFACE(linop_data_t);
+
+	struct wavelet_plan_s* plan;
+};
+
+DEF_TYPEID(wavelet_data_s);
+
+
+static void wavelet_normal(const linop_data_t* _data, data_t* out, const data_t* _in)
 {
-	struct wavelet_plan_s* plan = (struct wavelet_plan_s*) _data;
+	struct wavelet_plan_s* plan = CAST_DOWN(wavelet_data_s, _data)->plan;
+
 	md_copy(plan->numdims, plan->imSize, out, _in, sizeof(data_t));
 }
 
 
-/**
- * Wavelet linear operator
- *
- * @param numdims number of dimensions
- * @param imSize dimensions of x
- * @param wave_flags bitmask for Wavelet transform
- * @param minSize minimium size of coarse Wavelet scale
- * @param randshift apply random shift before Wavelet transforming
- * @param use_gpu true if using gpu
- */
-const struct linop_s* wavelet_create(int numdims, const long imSize[numdims], unsigned int wave_flags, const long minSize[numdims], bool randshift, bool use_gpu)
+static void wavelet_forward(const linop_data_t* _data, data_t* out, const data_t* _in)
 {
-	struct wavelet_plan_s* data = prepare_wavelet_plan(numdims, imSize, wave_flags, minSize, use_gpu);
-
-	data->randshift = randshift;
-
-	long coeff_dims[numdims];
-	md_select_dims( numdims, ~wave_flags, coeff_dims, imSize );
-	coeff_dims[0] = data->numCoeff_tr;
-	coeff_dims[1] = 1;
-	coeff_dims[2] = 1;
-
-	return linop_create(numdims, coeff_dims, numdims, imSize, data, wavelet_forward,  wavelet_inverse, wavelet_normal, NULL, wavelet_del);
-
-}
-
-
-/**
- * Proximal operator for l1-norm with Wavelet transform: f(x) = lambda || W x ||_1
- *
- * @param numdims number of dimensions
- * @param imSize dimensions of x
- * @param wave_flags bitmask for Wavelet transform
- * @param minSize minimium size of coarse Wavelet scale
- * @param lambda threshold parameter
- * @param randshift apply random shift before Wavelet transforming
- * @param use_gpu true if using gpu
- */
-const struct operator_p_s* prox_wavethresh_create(int numdims, const long imSize[numdims], unsigned int wave_flags, const long minSize[numdims], float lambda, bool randshift, bool use_gpu)
-{
-	struct wavelet_plan_s* data = prepare_wavelet_plan(numdims, imSize, wave_flags, minSize, use_gpu);
-
-	data->randshift = randshift;
-	data->lambda = lambda;
-
-	return operator_p_create(numdims, imSize, numdims, imSize, data, wavelet_thresh, wavelet_del);
-
-}
-
-void wavelet_forward(const void* _data, data_t* out, const data_t* _in)
-{
-	struct wavelet_plan_s* plan = (struct wavelet_plan_s*) _data;
+	struct wavelet_plan_s* plan = CAST_DOWN(wavelet_data_s, _data)->plan;
 
 	if (plan->randshift)
 		wavelet_new_randshift(plan);	
@@ -241,9 +208,9 @@ void wavelet_forward(const void* _data, data_t* out, const data_t* _in)
 }
 
 
-void wavelet_inverse(const void* _data, data_t* out, const data_t* _in)
+static void wavelet_inverse(const linop_data_t* _data, data_t* out, const data_t* _in)
 {
-	struct wavelet_plan_s* plan = (struct wavelet_plan_s*) _data;
+	struct wavelet_plan_s* plan = CAST_DOWN(wavelet_data_s, _data)->plan;
 	data_t* in = (data_t*) _in;
 
 	int numdims_tr = plan->numdims_tr;
@@ -275,6 +242,45 @@ void wavelet_inverse(const void* _data, data_t* out, const data_t* _in)
 }
 
 
+static void wavelet_del(const linop_data_t* _data)
+{
+	struct wavelet_data_s* data = CAST_DOWN(wavelet_data_s, _data);
+
+	// FIXME: free plan
+
+	free(data);
+}
+
+
+/**
+ * Wavelet linear operator
+ *
+ * @param numdims number of dimensions
+ * @param imSize dimensions of x
+ * @param wave_flags bitmask for Wavelet transform
+ * @param minSize minimium size of coarse Wavelet scale
+ * @param randshift apply random shift before Wavelet transforming
+ * @param use_gpu true if using gpu
+ */
+const struct linop_s* wavelet_create(int numdims, const long imSize[numdims], unsigned int wave_flags, const long minSize[numdims], bool randshift, bool use_gpu)
+{
+	PTR_ALLOC(struct wavelet_data_s, data);
+	SET_TYPEID(wavelet_data_s, data);
+
+	data->plan = prepare_wavelet_plan(numdims, imSize, wave_flags, minSize, use_gpu);
+
+	data->plan->randshift = randshift;
+
+	long coeff_dims[numdims];
+	md_select_dims(numdims, ~wave_flags, coeff_dims, imSize);
+	coeff_dims[0] = data->plan->numCoeff_tr;
+	coeff_dims[1] = 1;
+	coeff_dims[2] = 1;
+
+	return linop_create(numdims, coeff_dims, numdims, imSize, CAST_UP(PTR_PASS(data)), wavelet_forward, wavelet_inverse, wavelet_normal, NULL, wavelet_del);
+}
+
+
 
 void soft_thresh(struct wavelet_plan_s* plan, data_t* in, scalar_t thresh)
 {
@@ -292,9 +298,19 @@ void soft_thresh(struct wavelet_plan_s* plan, data_t* in, scalar_t thresh)
 	}
 }
 
-void wavelet_thresh(const void* _data, scalar_t thresh,  data_t* out, const data_t* _in)
+
+struct wave_prox_s {
+
+	INTERFACE(operator_data_t);
+	struct wavelet_plan_s* plan;
+};
+
+DEF_TYPEID(wave_prox_s);
+
+
+static void wavelet_thresh(const operator_data_t* _data, scalar_t thresh,  data_t* out, const data_t* _in)
 {
-	struct wavelet_plan_s* plan = (struct wavelet_plan_s*) _data;
+	struct wavelet_plan_s* plan = CAST_DOWN(wave_prox_s, _data)->plan;
 
 	if (plan->randshift)
 		wavelet_new_randshift(plan);
@@ -373,9 +389,38 @@ void wavelet_free(const struct wavelet_plan_s* plan)
 	}
 }
 
-static void wavelet_del(const void* data)
+
+
+static void wavelet_prox_del(const operator_data_t* _data)
 {
-	wavelet_free(data);
+	struct wave_prox_s* data = CAST_DOWN(wave_prox_s, _data);
+	wavelet_free(data->plan);
+	free(data);
+}
+
+/**
+ * Proximal operator for l1-norm with Wavelet transform: f(x) = lambda || W x ||_1
+ *
+ * @param numdims number of dimensions
+ * @param imSize dimensions of x
+ * @param wave_flags bitmask for Wavelet transform
+ * @param minSize minimium size of coarse Wavelet scale
+ * @param lambda threshold parameter
+ * @param randshift apply random shift before Wavelet transforming
+ * @param use_gpu true if using gpu
+ */
+const struct operator_p_s* prox_wavethresh_create(int numdims, const long imSize[numdims], unsigned int wave_flags, const long minSize[numdims], float lambda, bool randshift, bool use_gpu)
+{
+	PTR_ALLOC(struct wave_prox_s, data);
+	SET_TYPEID(wave_prox_s, data);
+
+	data->plan = prepare_wavelet_plan(numdims, imSize, wave_flags, minSize, use_gpu);
+
+	data->plan->randshift = randshift;
+	data->plan->lambda = lambda;
+
+	return operator_p_create(numdims, imSize, numdims, imSize, CAST_UP(PTR_PASS(data)), wavelet_thresh, wavelet_prox_del);
+
 }
 
 void fwt2_cpu(struct wavelet_plan_s* plan, data_t* coeff, data_t* inImage)
